from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Outline(Serialisable):
    tagname: str
    applyStyles: Any
    summaryBelow: Any
    summaryRight: Any
    showOutlineSymbols: Any
    def __init__(
        self,
        applyStyles: Any | None = ...,
        summaryBelow: Any | None = ...,
        summaryRight: Any | None = ...,
        showOutlineSymbols: Any | None = ...,
    ) -> None: ...

class PageSetupProperties(Serialisable):
    tagname: str
    autoPageBreaks: Any
    fitToPage: Any
    def __init__(self, autoPageBreaks: Any | None = ..., fitToPage: Any | None = ...) -> None: ...

class WorksheetProperties(Serialisable):
    tagname: str
    codeName: Any
    enableFormatConditionsCalculation: Any
    filterMode: Any
    published: Any
    syncHorizontal: Any
    syncRef: Any
    syncVertical: Any
    transitionEvaluation: Any
    transitionEntry: Any
    tabColor: Any
    outlinePr: Any
    pageSetUpPr: Any
    __elements__: Any
    def __init__(
        self,
        codeName: Any | None = ...,
        enableFormatConditionsCalculation: Any | None = ...,
        filterMode: Any | None = ...,
        published: Any | None = ...,
        syncHorizontal: Any | None = ...,
        syncRef: Any | None = ...,
        syncVertical: Any | None = ...,
        transitionEvaluation: Any | None = ...,
        transitionEntry: Any | None = ...,
        tabColor: Any | None = ...,
        outlinePr: Any | None = ...,
        pageSetUpPr: Any | None = ...,
    ) -> None: ...



import socket
from typing import Any

PROXY_TYPE_SOCKS4: int
PROXY_TYPE_SOCKS5: int
PROXY_TYPE_HTTP: int
PROXY_TYPE_HTTP_NO_TUNNEL: int

class ProxyError(Exception): ...
class GeneralProxyError(ProxyError): ...
class Socks5AuthError(ProxyError): ...
class Socks5Error(ProxyError): ...
class Socks4Error(ProxyError): ...
class HTTPError(ProxyError): ...

def setdefaultproxy(
    proxytype: Any | None = ...,
    addr: Any | None = ...,
    port: Any | None = ...,
    rdns: bool = ...,
    username: Any | None = ...,
    password: Any | None = ...,
) -> None: ...
def wrapmodule(module) -> None: ...

class socksocket(socket.socket):
    def __init__(self, family=..., type=..., proto: int = ..., _sock: Any | None = ...) -> None: ...
    def sendall(self, content, *args): ...
    def setproxy(
        self,
        proxytype: Any | None = ...,
        addr: Any | None = ...,
        port: Any | None = ...,
        rdns: bool = ...,
        username: Any | None = ...,
        password: Any | None = ...,
        headers: Any | None = ...,
    ) -> None: ...
    def getproxysockname(self): ...
    def getproxypeername(self): ...
    def getpeername(self): ...
    def connect(self, destpair) -> None: ...



from typing import Any

from ..exceptions.exceptions import FacadeSegmentMutationException as FacadeSegmentMutationException
from .segment import Segment as Segment

MUTATION_UNSUPPORTED_MESSAGE: str

class FacadeSegment(Segment):
    initializing: Any
    def __init__(self, name, entityid, traceid, sampled) -> None: ...
    def close(self, end_time: Any | None = ...) -> None: ...
    def put_http_meta(self, key, value) -> None: ...
    def put_annotation(self, key, value) -> None: ...
    def put_metadata(self, key, value, namespace: str = ...) -> None: ...
    def set_aws(self, aws_meta) -> None: ...
    def set_user(self, user) -> None: ...
    def add_throttle_flag(self) -> None: ...
    def add_fault_flag(self) -> None: ...
    def add_error_flag(self) -> None: ...
    def add_exception(self, exception, stack, remote: bool = ...) -> None: ...
    def apply_status_code(self, status_code) -> None: ...
    def serialize(self) -> None: ...
    def ready_to_send(self): ...
    def increment(self) -> None: ...
    def decrement_ref_counter(self) -> None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class TrendlineLabel(Serialisable):
    tagname: str
    layout: Any
    tx: Any
    numFmt: Any
    spPr: Any
    graphicalProperties: Any
    txPr: Any
    textProperties: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        layout: Any | None = ...,
        tx: Any | None = ...,
        numFmt: Any | None = ...,
        spPr: Any | None = ...,
        txPr: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class Trendline(Serialisable):
    tagname: str
    name: Any
    spPr: Any
    graphicalProperties: Any
    trendlineType: Any
    order: Any
    period: Any
    forward: Any
    backward: Any
    intercept: Any
    dispRSqr: Any
    dispEq: Any
    trendlineLbl: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        name: Any | None = ...,
        spPr: Any | None = ...,
        trendlineType: str = ...,
        order: Any | None = ...,
        period: Any | None = ...,
        forward: Any | None = ...,
        backward: Any | None = ...,
        intercept: Any | None = ...,
        dispRSqr: Any | None = ...,
        dispEq: Any | None = ...,
        trendlineLbl: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...



import builtins
import collections
import contextlib
import itertools
import operator
import pickle as pickle
import threading as threading
from abc import ABC as ABC
from datetime import timezone as timezone
from functools import reduce as reduce
from io import BytesIO as BytesIO, StringIO as StringIO
from itertools import zip_longest as zip_longest
from time import perf_counter as perf_counter
from typing import TYPE_CHECKING as TYPE_CHECKING, Any, NamedTuple
from urllib.parse import (
    parse_qsl as parse_qsl,
    quote as quote,
    quote_plus as quote_plus,
    unquote as unquote,
    unquote_plus as unquote_plus,
)

byte_buffer = BytesIO

py39: Any
py38: Any
py37: Any
py3k: Any
py2k: Any
pypy: Any
cpython: Any
win32: Any
osx: Any
arm: Any
has_refcount_gc: Any
contextmanager = contextlib.contextmanager
dottedgetter = operator.attrgetter
namedtuple = collections.namedtuple  # noqa: Y024
next = builtins.next

class FullArgSpec(NamedTuple):
    args: Any
    varargs: Any
    varkw: Any
    defaults: Any
    kwonlyargs: Any
    kwonlydefaults: Any
    annotations: Any

class nullcontext:
    enter_result: Any
    def __init__(self, enter_result: Any | None = ...) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *excinfo) -> None: ...

def inspect_getfullargspec(func): ...
def importlib_metadata_get(group): ...

string_types: tuple[type, ...]
binary_types: tuple[type, ...]
binary_type = bytes
text_type = str
int_types: tuple[type, ...]
iterbytes = iter
long_type = int
itertools_filterfalse = itertools.filterfalse
itertools_filter = filter
itertools_imap = map
exec_: Any
import_: Any
print_: Any

def b(s): ...
def b64decode(x): ...
def b64encode(x): ...
def decode_backslashreplace(text, encoding): ...
def cmp(a, b): ...
def raise_(exception, with_traceback: Any | None = ..., replace_context: Any | None = ..., from_: bool = ...) -> None: ...
def u(s): ...
def ue(s): ...

callable = builtins.callable

def safe_bytestring(text): ...
def inspect_formatargspec(
    args,
    varargs: Any | None = ...,
    varkw: Any | None = ...,
    defaults: Any | None = ...,
    kwonlyargs=...,
    kwonlydefaults=...,
    annotations=...,
    formatarg=...,
    formatvarargs=...,
    formatvarkw=...,
    formatvalue=...,
    formatreturns=...,
    formatannotation=...,
): ...
def dataclass_fields(cls): ...
def local_dataclass_fields(cls): ...
def raise_from_cause(exception, exc_info: Any | None = ...) -> None: ...
def reraise(tp, value, tb: Any | None = ..., cause: Any | None = ...) -> None: ...
def with_metaclass(meta, *bases, **kw): ...



from _typeshed import Incomplete

import _win32typing
from win32.lib.pywintypes import error as error

def AreFileApisANSI(): ...
def CancelIo(handle: int) -> None: ...
def CopyFile(_from: str, to: str, bFailIfExists) -> None: ...
def CopyFileW(_from: str, to: str, bFailIfExists) -> None: ...
def CreateDirectory(name: str, sa: _win32typing.PySECURITY_ATTRIBUTES) -> None: ...
def CreateDirectoryW(name: str, sa: _win32typing.PySECURITY_ATTRIBUTES) -> None: ...
def CreateDirectoryEx(templateName: str, newDirectory: str, sa: _win32typing.PySECURITY_ATTRIBUTES) -> None: ...
def CreateFile(
    fileName: str,
    desiredAccess,
    shareMode,
    attributes: _win32typing.PySECURITY_ATTRIBUTES,
    CreationDisposition,
    flagsAndAttributes,
    hTemplateFile: int,
) -> int: ...
def CreateIoCompletionPort(handle: int, existing: int, completionKey, numThreads) -> int: ...
def CreateMailslot(Name, MaxMessageSize, ReadTimeout, SecurityAttributes: _win32typing.PySECURITY_ATTRIBUTES) -> int: ...
def GetMailslotInfo(Mailslot: int) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def SetMailslotInfo(Mailslot: int, ReadTimeout) -> None: ...
def DefineDosDevice(flags, deviceName: str, targetPath: str) -> None: ...
def DefineDosDeviceW(flags, deviceName: str, targetPath: str) -> None: ...
def DeleteFile(fileName: str) -> None: ...
def DeviceIoControl(Device: int, IoControlCode, InBuffer, OutBuffer, Overlapped: _win32typing.PyOVERLAPPED | None = ...): ...
def FindClose(hFindFile) -> None: ...
def FindCloseChangeNotification(hChangeHandle) -> None: ...
def FindFirstChangeNotification(pathName: str, bWatchSubtree, notifyFilter): ...
def FindNextChangeNotification(hChangeHandle): ...
def FlushFileBuffers(hFile: int) -> None: ...
def GetBinaryType(appName: str): ...
def GetDiskFreeSpace(rootPathName: str) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
def GetDiskFreeSpaceEx(rootPathName: str) -> tuple[Incomplete, Incomplete, Incomplete]: ...
def GetDriveType(rootPathName: str): ...
def GetDriveTypeW(rootPathName: str): ...
def GetFileAttributes(fileName: str): ...
def GetFileAttributesW(fileName: str): ...
def GetFileTime(
    handle: int, creationTime: _win32typing.PyTime, accessTime: _win32typing.PyTime, writeTime: _win32typing.PyTime
) -> tuple[_win32typing.PyTime, _win32typing.PyTime, _win32typing.PyTime]: ...
def SetFileTime(
    File: int,
    CreationTime: _win32typing.PyTime | None = ...,
    LastAccessTime: _win32typing.PyTime | None = ...,
    LastWriteTime: _win32typing.PyTime | None = ...,
    UTCTimes: bool = ...,
) -> None: ...
def GetFileInformationByHandle(handle: int): ...
def GetCompressedFileSize(): ...
def GetFileSize(): ...
def AllocateReadBuffer(bufSize) -> _win32typing.PyOVERLAPPEDReadBuffer: ...
def ReadFile(
    hFile: int, buffer_bufSize: _win32typing.PyOVERLAPPEDReadBuffer, overlapped: _win32typing.PyOVERLAPPED | None = ...
) -> tuple[Incomplete, str]: ...
def WriteFile(
    hFile: int, data: str | _win32typing.PyOVERLAPPEDReadBuffer, ol: _win32typing.PyOVERLAPPED | None = ...
) -> tuple[Incomplete, Incomplete]: ...
def CloseHandle(handle: int) -> None: ...
def LockFileEx(hFile: int, _int, _int1, _int2, ol: _win32typing.PyOVERLAPPED | None = ...) -> None: ...
def UnlockFileEx(hFile: int, _int, _int1, ol: _win32typing.PyOVERLAPPED | None = ...) -> None: ...
def GetQueuedCompletionStatus(hPort: int, timeOut) -> tuple[Incomplete, Incomplete, Incomplete, _win32typing.PyOVERLAPPED]: ...
def PostQueuedCompletionStatus(
    handle: int, numberOfbytes: int = ..., completionKey: int = ..., overlapped: _win32typing.PyOVERLAPPED | None = ...
): ...
def GetFileType(hFile: int): ...
def GetLogicalDrives(): ...
def GetOverlappedResult(hFile: int, overlapped: _win32typing.PyOVERLAPPED, bWait): ...
def LockFile(hFile: int, offsetLow, offsetHigh, nNumberOfBytesToLockLow, nNumberOfBytesToLockHigh) -> None: ...
def MoveFile(existingFileName: str, newFileName: str) -> None: ...
def MoveFileW(existingFileName: str, newFileName: str) -> None: ...
def MoveFileEx(existingFileName: str, newFileName: str, flags) -> None: ...
def MoveFileExW(existingFileName: str, newFileName: str, flags) -> None: ...
def QueryDosDevice(DeviceName: str) -> str: ...
def ReadDirectoryChangesW(
    handle: int, size, bWatchSubtree, dwNotifyFilter, overlapped: _win32typing.PyOVERLAPPED | None = ...
) -> None: ...
def FILE_NOTIFY_INFORMATION(buffer: str, size) -> tuple[tuple[Incomplete, Incomplete], ...]: ...
def SetCurrentDirectory(lpPathName: str) -> None: ...
def SetEndOfFile(hFile: int) -> None: ...
def SetFileApisToANSI() -> None: ...
def SetFileApisToOEM() -> None: ...
def SetFileAttributes(filename: str, newAttributes) -> None: ...
def SetFilePointer(handle: int, offset, moveMethod) -> None: ...
def SetVolumeLabel(rootPathName: str, volumeName: str) -> None: ...
def UnlockFile(hFile: int, offsetLow, offsetHigh, nNumberOfBytesToUnlockLow, nNumberOfBytesToUnlockHigh) -> None: ...
def TransmitFile(
    Socket,
    File: int,
    NumberOfBytesToWrite,
    NumberOfBytesPerSend,
    Overlapped: _win32typing.PyOVERLAPPED,
    Flags,
    Head: Incomplete | None = ...,
    Tail: Incomplete | None = ...,
) -> None: ...
def ConnectEx(
    s, name, Overlapped: _win32typing.PyOVERLAPPED, SendBuffer: Incomplete | None = ...
) -> tuple[Incomplete, Incomplete]: ...
def AcceptEx(slistening, sAccepting, buffer, ol: _win32typing.PyOVERLAPPED) -> None: ...
def CalculateSocketEndPointSize(socket): ...
def GetAcceptExSockaddrs(
    sAccepting, buffer: _win32typing.PyOVERLAPPEDReadBuffer
) -> tuple[Incomplete, Incomplete, Incomplete]: ...
def WSAEventSelect(socket, hEvent, networkEvents) -> None: ...
def WSAEnumNetworkEvents(s, hEvent: int): ...
def WSAAsyncSelect(socket, hwnd: int, _int, networkEvents) -> None: ...
def WSASend(s, buffer: str, ol: _win32typing.PyOVERLAPPED, dwFlags) -> tuple[Incomplete, Incomplete]: ...
def WSARecv(s, buffer, ol: _win32typing.PyOVERLAPPED, dwFlags) -> tuple[Incomplete, Incomplete]: ...
def BuildCommDCB(_def: str, dcb: _win32typing.PyDCB) -> _win32typing.PyDCB: ...
def ClearCommError(PyHANDLE) -> tuple[Incomplete, _win32typing.PyCOMSTAT]: ...
def EscapeCommFunction(handle: int) -> None: ...
def GetCommState(handle: int) -> _win32typing.PyDCB: ...
def SetCommState(handle: int, dcb: _win32typing.PyDCB) -> None: ...
def ClearCommBreak(handle: int) -> None: ...
def GetCommMask(handle: int): ...
def SetCommMask(handle: int, val): ...
def GetCommModemStatus(handle: int): ...
def GetCommTimeouts(handle: int): ...
def SetCommTimeouts(handle: int, val): ...
def PurgeComm(handle: int, action) -> None: ...
def SetCommBreak(handle: int) -> None: ...
def SetupComm(handle: int, dwInQueue, dwOutQueue) -> None: ...
def TransmitCommChar(handle: int, cChar) -> None: ...
def WaitCommEvent(handle: int, overlapped: _win32typing.PyOVERLAPPED) -> None: ...
def SetVolumeMountPoint(VolumeMountPoint: str, VolumeName: str) -> str: ...
def DeleteVolumeMountPoint(VolumeMountPoint: str) -> None: ...
def GetVolumeNameForVolumeMountPoint(VolumeMountPoint: str) -> str: ...
def GetVolumePathName(FileName: str, BufferLength: int = ...) -> str: ...
def GetVolumePathNamesForVolumeName(VolumeName: str) -> list[Incomplete]: ...
def CreateHardLink(
    FileName: str,
    ExistingFileName: str,
    SecurityAttributes: _win32typing.PySECURITY_ATTRIBUTES | None = ...,
    Transaction: int | None = ...,
) -> None: ...
def CreateSymbolicLink(SymlinkFileName: str, TargetFileName: str, Flags: int = ..., Transaction: int | None = ...) -> None: ...
def EncryptFile(filename: str) -> None: ...
def DecryptFile(filename: str) -> None: ...
def EncryptionDisable(DirName: str, Disable) -> None: ...
def FileEncryptionStatus(FileName: str): ...
def QueryUsersOnEncryptedFile(FileName: str) -> tuple[_win32typing.PySID, str, Incomplete]: ...
def QueryRecoveryAgentsOnEncryptedFile(FileName: str) -> tuple[_win32typing.PySID, str, Incomplete]: ...
def RemoveUsersFromEncryptedFile(FileName: str, pHashes: tuple[tuple[_win32typing.PySID, str, Incomplete], ...]) -> None: ...
def AddUsersToEncryptedFile(FileName: str, pUsers: tuple[tuple[_win32typing.PySID, str, Incomplete], ...]) -> None: ...
def DuplicateEncryptionInfoFile(
    SrcFileName: str,
    DstFileName: str,
    CreationDisposition,
    Attributes,
    SecurityAttributes: _win32typing.PySECURITY_ATTRIBUTES | None = ...,
) -> None: ...
def BackupRead(
    hFile: int, NumberOfBytesToRead, Buffer, bAbort, bProcessSecurity, lpContext
) -> tuple[Incomplete, Incomplete, Incomplete]: ...
def BackupSeek(hFile: int, NumberOfBytesToSeek, lpContext): ...
def BackupWrite(
    hFile: int, NumberOfBytesToWrite, Buffer: str, bAbort, bProcessSecurity, lpContext
) -> tuple[Incomplete, Incomplete]: ...
def SetFileShortName(hFile: int, ShortName) -> None: ...
def CopyFileEx(
    ExistingFileName,
    NewFileName,
    ProgressRoutine: _win32typing.CopyProgressRoutine | None = ...,
    Data: Incomplete | None = ...,
    Cancel: bool = ...,
    CopyFlags: int = ...,
    Transaction: int | None = ...,
) -> None: ...
def MoveFileWithProgress(
    ExistingFileName,
    NewFileName,
    ProgressRoutine: _win32typing.CopyProgressRoutine | None = ...,
    Data: Incomplete | None = ...,
    Flags: int = ...,
    Transaction: int | None = ...,
) -> None: ...
def ReplaceFile(
    ReplacedFileName,
    ReplacementFileName,
    BackupFileName: Incomplete | None = ...,
    ReplaceFlags: int = ...,
    Exclude: Incomplete | None = ...,
    Reserved: Incomplete | None = ...,
) -> None: ...
def OpenEncryptedFileRaw(FileName, Flags): ...
def ReadEncryptedFileRaw(ExportCallback, CallbackContext, Context) -> None: ...
def WriteEncryptedFileRaw(ImportCallback, CallbackContext, Context) -> None: ...
def CloseEncryptedFileRaw(Context) -> None: ...
def CreateFileW(
    FileName: str,
    DesiredAccess,
    ShareMode,
    SecurityAttributes: _win32typing.PySECURITY_ATTRIBUTES,
    CreationDisposition,
    FlagsAndAttributes,
    TemplateFile: int | None = ...,
    Transaction: int | None = ...,
    MiniVersion: Incomplete | None = ...,
    ExtendedParameter: Incomplete | None = ...,
) -> int: ...
def DeleteFileW(FileName: str, Transaction: int | None = ...) -> None: ...
def GetFileAttributesEx(FileName: str, InfoLevelId, Transaction: int | None = ...): ...
def SetFileAttributesW(FileName, FileAttributes, Transaction: int | None = ...) -> None: ...
def CreateDirectoryExW(
    TemplateDirectory: str,
    NewDirectory: str,
    SecurityAttributes: _win32typing.PySECURITY_ATTRIBUTES | None = ...,
    Transaction: int | None = ...,
) -> None: ...
def RemoveDirectory(PathName: str, Transaction: int | None = ...) -> None: ...
def FindFilesW(FileName: str, Transaction: int | None = ...): ...
def FindFilesIterator(FileName: str, Transaction: int | None = ...): ...
def FindStreams(FileName: str, Transaction: int | None = ...) -> list[tuple[Incomplete, str]]: ...
def FindFileNames(FileName: str, Transaction: int | None = ...) -> list[Incomplete]: ...
def GetFinalPathNameByHandle(File: int, Flags) -> str: ...
def SfcGetNextProtectedFile() -> list[Incomplete]: ...
def SfcIsFileProtected(ProtFileName: str): ...
def GetLongPathName(ShortPath: str, Transaction: int | None = ...) -> str: ...
def GetFullPathName(FileName, Transaction: int | None = ...): ...
def Wow64DisableWow64FsRedirection(): ...
def Wow64RevertWow64FsRedirection(OldValue) -> None: ...
def GetFileInformationByHandleEx(File: int, FileInformationClass): ...
def SetFileInformationByHandle(File: int, FileInformationClass, Information) -> None: ...
def ReOpenFile(OriginalFile: int, DesiredAccess, ShareMode, Flags) -> int: ...
def OpenFileById(
    File: int,
    FileId: _win32typing.PyIID,
    DesiredAccess,
    ShareMode,
    Flags,
    SecurityAttributes: _win32typing.PySECURITY_ATTRIBUTES | None = ...,
) -> int: ...
def DCB(*args, **kwargs): ...  # incomplete
def GetFileAttributesExW(*args, **kwargs): ...  # incomplete
def OVERLAPPED(*args, **kwargs): ...  # incomplete

CALLBACK_CHUNK_FINISHED: int
CALLBACK_STREAM_SWITCH: int
CBR_110: int
CBR_115200: int
CBR_1200: int
CBR_128000: int
CBR_14400: int
CBR_19200: int
CBR_2400: int
CBR_256000: int
CBR_300: int
CBR_38400: int
CBR_4800: int
CBR_56000: int
CBR_57600: int
CBR_600: int
CBR_9600: int
CLRBREAK: int
CLRDTR: int
CLRRTS: int
COPY_FILE_ALLOW_DECRYPTED_DESTINATION: int
COPY_FILE_FAIL_IF_EXISTS: int
COPY_FILE_OPEN_SOURCE_FOR_WRITE: int
COPY_FILE_RESTARTABLE: int
CREATE_ALWAYS: int
CREATE_FOR_DIR: int
CREATE_FOR_IMPORT: int
CREATE_NEW: int
DRIVE_CDROM: int
DRIVE_FIXED: int
DRIVE_NO_ROOT_DIR: int
DRIVE_RAMDISK: int
DRIVE_REMOTE: int
DRIVE_REMOVABLE: int
DRIVE_UNKNOWN: int
DTR_CONTROL_DISABLE: int
DTR_CONTROL_ENABLE: int
DTR_CONTROL_HANDSHAKE: int
EV_BREAK: int
EV_CTS: int
EV_DSR: int
EV_ERR: int
EV_RING: int
EV_RLSD: int
EV_RXCHAR: int
EV_RXFLAG: int
EV_TXEMPTY: int
EVENPARITY: int
FD_ACCEPT: int
FD_CLOSE: int
FD_CONNECT: int
FD_GROUP_QOS: int
FD_OOB: int
FD_QOS: int
FD_READ: int
FD_ROUTING_INTERFACE_CHANGE: int
FD_WRITE: int
FILE_ALL_ACCESS: int
FILE_ATTRIBUTE_ARCHIVE: int
FILE_ATTRIBUTE_COMPRESSED: int
FILE_ATTRIBUTE_DIRECTORY: int
FILE_ATTRIBUTE_HIDDEN: int
FILE_ATTRIBUTE_NORMAL: int
FILE_ATTRIBUTE_OFFLINE: int
FILE_ATTRIBUTE_READONLY: int
FILE_ATTRIBUTE_SYSTEM: int
FILE_ATTRIBUTE_TEMPORARY: int
FILE_BEGIN: int
FILE_CURRENT: int
FILE_ENCRYPTABLE: int
FILE_END: int
FILE_FLAG_BACKUP_SEMANTICS: int
FILE_FLAG_DELETE_ON_CLOSE: int
FILE_FLAG_NO_BUFFERING: int
FILE_FLAG_OPEN_REPARSE_POINT: int
FILE_FLAG_OVERLAPPED: int
FILE_FLAG_POSIX_SEMANTICS: int
FILE_FLAG_RANDOM_ACCESS: int
FILE_FLAG_SEQUENTIAL_SCAN: int
FILE_FLAG_WRITE_THROUGH: int
FILE_GENERIC_READ: int
FILE_GENERIC_WRITE: int
FILE_IS_ENCRYPTED: int
FILE_READ_ONLY: int
FILE_ROOT_DIR: int
FILE_SHARE_DELETE: int
FILE_SHARE_READ: int
FILE_SHARE_WRITE: int
FILE_SYSTEM_ATTR: int
FILE_SYSTEM_DIR: int
FILE_SYSTEM_NOT_SUPPORT: int
FILE_TYPE_CHAR: int
FILE_TYPE_DISK: int
FILE_TYPE_PIPE: int
FILE_TYPE_UNKNOWN: int
FILE_UNKNOWN: int
FILE_USER_DISALLOWED: int
FileAllocationInfo: int
FileAttributeTagInfo: int
FileBasicInfo: int
FileCompressionInfo: int
FileDispositionInfo: int
FileEndOfFileInfo: int
FileIdBothDirectoryInfo: int
FileIdBothDirectoryRestartInfo: int
FileIdType: int
FileIoPriorityHintInfo: int
FileNameInfo: int
FileRenameInfo: int
FileStandardInfo: int
FileStreamInfo: int
GENERIC_EXECUTE: int
GENERIC_READ: int
GENERIC_WRITE: int
GetFileExInfoStandard: int
IoPriorityHintLow: int
IoPriorityHintNormal: int
IoPriorityHintVeryLow: int
MARKPARITY: int
MOVEFILE_COPY_ALLOWED: int
MOVEFILE_CREATE_HARDLINK: int
MOVEFILE_DELAY_UNTIL_REBOOT: int
MOVEFILE_FAIL_IF_NOT_TRACKABLE: int
MOVEFILE_REPLACE_EXISTING: int
MOVEFILE_WRITE_THROUGH: int
NOPARITY: int
ObjectIdType: int
ODDPARITY: int
ONE5STOPBITS: int
ONESTOPBIT: int
OPEN_ALWAYS: int
OPEN_EXISTING: int
OVERWRITE_HIDDEN: int
PROGRESS_CANCEL: int
PROGRESS_CONTINUE: int
PROGRESS_QUIET: int
PROGRESS_STOP: int
PURGE_RXABORT: int
PURGE_RXCLEAR: int
PURGE_TXABORT: int
PURGE_TXCLEAR: int
REPLACEFILE_IGNORE_MERGE_ERRORS: int
REPLACEFILE_WRITE_THROUGH: int
RTS_CONTROL_DISABLE: int
RTS_CONTROL_ENABLE: int
RTS_CONTROL_HANDSHAKE: int
RTS_CONTROL_TOGGLE: int
SCS_32BIT_BINARY: int
SCS_DOS_BINARY: int
SCS_OS216_BINARY: int
SCS_PIF_BINARY: int
SCS_POSIX_BINARY: int
SCS_WOW_BINARY: int
SECURITY_ANONYMOUS: int
SECURITY_CONTEXT_TRACKING: int
SECURITY_DELEGATION: int
SECURITY_EFFECTIVE_ONLY: int
SECURITY_IDENTIFICATION: int
SECURITY_IMPERSONATION: int
SETBREAK: int
SETDTR: int
SETRTS: int
SETXOFF: int
SETXON: int
SO_CONNECT_TIME: int
SO_UPDATE_ACCEPT_CONTEXT: int
SO_UPDATE_CONNECT_CONTEXT: int
SPACEPARITY: int
SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE: int
SYMBOLIC_LINK_FLAG_DIRECTORY: int
TF_DISCONNECT: int
TF_REUSE_SOCKET: int
TF_USE_DEFAULT_WORKER: int
TF_USE_KERNEL_APC: int
TF_USE_SYSTEM_THREAD: int
TF_WRITE_BEHIND: int
TRUNCATE_EXISTING: int
TWOSTOPBITS: int
WSA_IO_PENDING: int
WSA_OPERATION_ABORTED: int
WSAECONNABORTED: int
WSAECONNRESET: int
WSAEDISCON: int
WSAEFAULT: int
WSAEINPROGRESS: int
WSAEINTR: int
WSAEINVAL: int
WSAEMSGSIZE: int
WSAENETDOWN: int
WSAENETRESET: int
WSAENOBUFS: int
WSAENOTCONN: int
WSAENOTSOCK: int
WSAEOPNOTSUPP: int
WSAESHUTDOWN: int
WSAEWOULDBLOCK: int
FD_ADDRESS_LIST_CHANGE: int
INVALID_HANDLE_VALUE: int
UNICODE: int



from .cluster import RedisClusterCommands as RedisClusterCommands
from .core import AsyncCoreCommands as AsyncCoreCommands, CoreCommands as CoreCommands
from .helpers import list_or_args as list_or_args
from .parser import CommandsParser as CommandsParser
from .redismodules import RedisModuleCommands as RedisModuleCommands
from .sentinel import AsyncSentinelCommands as AsyncSentinelCommands, SentinelCommands as SentinelCommands

__all__ = [
    "RedisClusterCommands",
    "CommandsParser",
    "AsyncCoreCommands",
    "CoreCommands",
    "list_or_args",
    "RedisModuleCommands",
    "AsyncSentinelCommands",
    "SentinelCommands",
]



from _typeshed import SupportsWrite
from collections.abc import Callable, Iterable, Mapping, MutableMapping
from typing import Any, Generic, TypeVar, overload

_MappingT = TypeVar("_MappingT", bound=Mapping[str, Any])

def dump(o: _MappingT, f: SupportsWrite[str], encoder: TomlEncoder[_MappingT] | None = ...) -> str: ...
def dumps(o: _MappingT, encoder: TomlEncoder[_MappingT] | None = ...) -> str: ...

class TomlEncoder(Generic[_MappingT]):
    _dict: type[_MappingT]
    preserve: bool
    dump_funcs: MutableMapping[type[Any], Callable[[Any], str]]
    @overload
    def __init__(self, _dict: type[_MappingT], preserve: bool = ...) -> None: ...
    @overload
    def __init__(self: TomlEncoder[dict[str, Any]], _dict: type[dict[str, Any]] = ..., preserve: bool = ...) -> None: ...
    def get_empty_table(self) -> _MappingT: ...
    def dump_list(self, v: Iterable[object]) -> str: ...
    def dump_inline_table(self, section: dict[str, Any] | Any) -> str: ...
    def dump_value(self, v: Any) -> str: ...
    def dump_sections(self, o: _MappingT, sup: str) -> tuple[str, _MappingT]: ...

class TomlPreserveInlineDictEncoder(TomlEncoder[_MappingT]):
    @overload
    def __init__(self, _dict: type[_MappingT]) -> None: ...
    @overload
    def __init__(self: TomlPreserveInlineDictEncoder[dict[str, Any]], _dict: type[dict[str, Any]] = ...) -> None: ...

class TomlArraySeparatorEncoder(TomlEncoder[_MappingT]):
    separator: str
    @overload
    def __init__(self, _dict: type[_MappingT], preserve: bool = ..., separator: str = ...) -> None: ...
    @overload
    def __init__(
        self: TomlArraySeparatorEncoder[dict[str, Any]],
        _dict: type[dict[str, Any]] = ...,
        preserve: bool = ...,
        separator: str = ...,
    ) -> None: ...
    def dump_list(self, v: Iterable[Any]) -> str: ...

class TomlNumpyEncoder(TomlEncoder[_MappingT]): ...
class TomlPreserveCommentEncoder(TomlEncoder[_MappingT]): ...
class TomlPathlibEncoder(TomlEncoder[_MappingT]): ...



def newer(source: str, target: str) -> bool: ...
def newer_pairwise(sources: list[str], targets: list[str]) -> list[tuple[str, str]]: ...
def newer_group(sources: list[str], target: str, missing: str = ...) -> bool: ...



from typing import Any

class RequestValidator:
    def __init__(self) -> None: ...
    @property
    def allowed_signature_methods(self): ...
    @property
    def safe_characters(self): ...
    @property
    def client_key_length(self): ...
    @property
    def request_token_length(self): ...
    @property
    def access_token_length(self): ...
    @property
    def timestamp_lifetime(self): ...
    @property
    def nonce_length(self): ...
    @property
    def verifier_length(self): ...
    @property
    def realms(self): ...
    @property
    def enforce_ssl(self): ...
    def check_client_key(self, client_key): ...
    def check_request_token(self, request_token): ...
    def check_access_token(self, request_token): ...
    def check_nonce(self, nonce): ...
    def check_verifier(self, verifier): ...
    def check_realms(self, realms): ...
    @property
    def dummy_client(self) -> None: ...
    @property
    def dummy_request_token(self) -> None: ...
    @property
    def dummy_access_token(self) -> None: ...
    def get_client_secret(self, client_key, request) -> None: ...
    def get_request_token_secret(self, client_key, token, request) -> None: ...
    def get_access_token_secret(self, client_key, token, request) -> None: ...
    def get_default_realms(self, client_key, request) -> None: ...
    def get_realms(self, token, request) -> None: ...
    def get_redirect_uri(self, token, request) -> None: ...
    def get_rsa_key(self, client_key, request) -> None: ...
    def invalidate_request_token(self, client_key, request_token, request) -> None: ...
    def validate_client_key(self, client_key, request) -> None: ...
    def validate_request_token(self, client_key, token, request) -> None: ...
    def validate_access_token(self, client_key, token, request) -> None: ...
    def validate_timestamp_and_nonce(
        self, client_key, timestamp, nonce, request, request_token: Any | None = ..., access_token: Any | None = ...
    ) -> None: ...
    def validate_redirect_uri(self, client_key, redirect_uri, request) -> None: ...
    def validate_requested_realms(self, client_key, realms, request) -> None: ...
    def validate_realms(self, client_key, token, request, uri: Any | None = ..., realms: Any | None = ...) -> None: ...
    def validate_verifier(self, client_key, token, verifier, request) -> None: ...
    def verify_request_token(self, token, request) -> None: ...
    def verify_realms(self, token, realms, request) -> None: ...
    def save_access_token(self, token, request) -> None: ...
    def save_request_token(self, token, request) -> None: ...
    def save_verifier(self, token, verifier, request) -> None: ...



import sys

from .universaldetector import UniversalDetector as UniversalDetector, _FinalResultType, _IntermediateResultType
from .version import VERSION as VERSION, __version__ as __version__

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

# unused in this module, but imported in multiple submodules
class _LangModelType(TypedDict):  # noqa: Y049
    char_to_order_map: tuple[int, ...]
    precedence_matrix: tuple[int, ...]
    typical_positive_ratio: float
    keep_english_letter: bool
    charset_name: str
    language: str

def detect(byte_str: bytes | bytearray) -> _FinalResultType: ...
def detect_all(byte_str: bytes | bytearray, ignore_threshold: bool = ...) -> list[_IntermediateResultType]: ...



from typing import Any

ANSI_CSI: str
ANSI_SGR: str
ANSI_ERASE_LINE: Any
ANSI_RESET: Any
ANSI_HIDE_CURSOR: Any
ANSI_SHOW_CURSOR: Any
ANSI_COLOR_CODES: Any
ANSI_TEXT_STYLES: Any
CLEAN_OUTPUT_PATTERN: Any
DEFAULT_LINES: int
DEFAULT_COLUMNS: int
DEFAULT_ENCODING: str
HIGHLIGHT_COLOR: Any

def ansi_strip(text, readline_hints: bool = ...): ...
def ansi_style(**kw): ...
def ansi_width(text): ...
def ansi_wrap(text, **kw): ...
def auto_encode(stream, text, *args, **kw) -> None: ...
def clean_terminal_output(text): ...
def connected_to_terminal(stream: Any | None = ...): ...
def enable_ansi_support(): ...
def find_terminal_size(): ...
def find_terminal_size_using_ioctl(stream): ...
def find_terminal_size_using_stty(): ...
def get_pager_command(text: Any | None = ...): ...
def have_windows_native_ansi_support(): ...
def message(text, *args, **kw) -> None: ...
def output(text, *args, **kw) -> None: ...
def readline_strip(expr): ...
def readline_wrap(expr): ...
def show_pager(formatted_text, encoding=...) -> None: ...
def terminal_supports_colors(stream: Any | None = ...): ...
def usage(usage_text) -> None: ...
def warning(text, *args, **kw) -> None: ...



from typing import Any, Generic

from redis.asyncio.client import Pipeline, Redis
from redis.client import _StrType

def from_url(url: str, **kwargs) -> Redis[Any]: ...

class pipeline(Generic[_StrType]):
    p: Pipeline[_StrType]
    def __init__(self, redis_obj: Redis[_StrType]) -> None: ...
    async def __aenter__(self) -> Pipeline[_StrType]: ...
    async def __aexit__(self, exc_type: object, exc_value: object, traceback: object) -> None: ...



from braintree.merchant_account.business_details import BusinessDetails as BusinessDetails
from braintree.merchant_account.funding_details import FundingDetails as FundingDetails
from braintree.merchant_account.individual_details import IndividualDetails as IndividualDetails
from braintree.merchant_account.merchant_account import MerchantAccount as MerchantAccount



from collections.abc import Generator
from typing import Any

from ..formatter import Formatter
from .bbcode import BBCodeFormatter as BBCodeFormatter
from .html import HtmlFormatter as HtmlFormatter
from .img import (
    BmpImageFormatter as BmpImageFormatter,
    GifImageFormatter as GifImageFormatter,
    ImageFormatter as ImageFormatter,
    JpgImageFormatter as JpgImageFormatter,
)
from .irc import IRCFormatter as IRCFormatter
from .latex import LatexFormatter as LatexFormatter
from .other import NullFormatter as NullFormatter, RawTokenFormatter as RawTokenFormatter, TestcaseFormatter as TestcaseFormatter
from .pangomarkup import PangoMarkupFormatter as PangoMarkupFormatter
from .rtf import RtfFormatter as RtfFormatter
from .svg import SvgFormatter as SvgFormatter
from .terminal import TerminalFormatter as TerminalFormatter
from .terminal256 import Terminal256Formatter as Terminal256Formatter, TerminalTrueColorFormatter as TerminalTrueColorFormatter

def get_all_formatters() -> Generator[type[Formatter[Any]], None, None]: ...
def get_formatter_by_name(_alias, **options): ...
def load_formatter_from_file(filename, formattername: str = ..., **options): ...
def get_formatter_for_filename(fn, **options): ...



from _heapq import *
from _typeshed import SupportsRichComparison
from collections.abc import Callable, Iterable
from typing import Any, TypeVar

__all__ = ["heappush", "heappop", "heapify", "heapreplace", "merge", "nlargest", "nsmallest", "heappushpop"]

_S = TypeVar("_S")

__about__: str

def merge(
    *iterables: Iterable[_S], key: Callable[[_S], SupportsRichComparison] | None = ..., reverse: bool = ...
) -> Iterable[_S]: ...
def nlargest(n: int, iterable: Iterable[_S], key: Callable[[_S], SupportsRichComparison] | None = ...) -> list[_S]: ...
def nsmallest(n: int, iterable: Iterable[_S], key: Callable[[_S], SupportsRichComparison] | None = ...) -> list[_S]: ...
def _heapify_max(__x: list[Any]) -> None: ...  # undocumented



import sys
from typing import Any
from typing_extensions import Literal

if sys.platform != "win32":
    from socket import AF_UNIX as AF_UNIX

unix_socket_available: bool

class Server:
    ipc: bool
    host: Any
    port: Any
    allowed_referral_hosts: Any
    ssl: Any
    tls: Any
    name: Any
    get_info: Any
    dit_lock: Any
    custom_formatter: Any
    custom_validator: Any
    current_address: Any
    connect_timeout: Any
    mode: Any
    def __init__(
        self,
        host: str,
        port: int | None = ...,
        use_ssl: bool = ...,
        allowed_referral_hosts: Any | None = ...,
        get_info: Literal["NO_INFO", "DSA", "SCHEMA", "ALL"] = ...,
        tls: Any | None = ...,
        formatter: Any | None = ...,
        connect_timeout: Any | None = ...,
        mode: Literal["IP_SYSTEM_DEFAULT", "IP_V4_ONLY", "IP_V6_ONLY", "IP_V4_PREFERRED", "IP_V6_PREFERRED"] = ...,
        validator: Any | None = ...,
    ) -> None: ...
    @property
    def address_info(self): ...
    def update_availability(self, address, available) -> None: ...
    def reset_availability(self) -> None: ...
    def check_availability(
        self, source_address: Any | None = ..., source_port: Any | None = ..., source_port_list: Any | None = ...
    ): ...
    @staticmethod
    def next_message_id(): ...
    def get_info_from_server(self, connection) -> None: ...
    def attach_dsa_info(self, dsa_info: Any | None = ...) -> None: ...
    def attach_schema_info(self, dsa_schema: Any | None = ...) -> None: ...
    @property
    def info(self): ...
    @property
    def schema(self): ...
    @staticmethod
    def from_definition(
        host,
        dsa_info,
        dsa_schema,
        port: Any | None = ...,
        use_ssl: bool = ...,
        formatter: Any | None = ...,
        validator: Any | None = ...,
    ): ...
    def candidate_addresses(self): ...
    def has_control(self, control): ...
    def has_extension(self, extension): ...
    def has_feature(self, feature): ...



import sys
import threading
from _typeshed import Self
from collections.abc import Callable
from queue import Queue
from types import ModuleType, TracebackType
from typing import Any, ClassVar, Generic, TypeVar
from typing_extensions import ParamSpec, TypedDict

_T = TypeVar("_T")
_AbstractListener_T = TypeVar("_AbstractListener_T", bound=AbstractListener)
_P = ParamSpec("_P")

class _RESOLUTIONS(TypedDict):
    darwin: str
    uinput: str
    xorg: str

RESOLUTIONS: _RESOLUTIONS

def backend(package: str) -> ModuleType: ...
def prefix(base: type | tuple[type | tuple[Any, ...], ...], cls: type) -> str | None: ...

class AbstractListener(threading.Thread):
    class StopException(Exception): ...
    _HANDLED_EXCEPTIONS: ClassVar[tuple[type | tuple[Any, ...], ...]]  # undocumented
    _suppress: bool  # undocumented
    _running: bool  # undocumented
    _thread: threading.Thread  # undocumented
    _condition: threading.Condition  # undocumented
    _ready: bool  # undocumented
    _queue: Queue[sys._OptExcInfo | None]  # undocumented
    daemon: bool
    def __init__(self, suppress: bool = ..., **kwargs: Callable[..., bool | None] | None) -> None: ...
    @property
    def suppress(self) -> bool: ...
    @property
    def running(self) -> bool: ...
    def stop(self) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def wait(self) -> None: ...
    def run(self) -> None: ...
    @classmethod
    def _emitter(cls, f: Callable[_P, _T]) -> Callable[_P, _T]: ...  # undocumented
    def _mark_ready(self) -> None: ...  # undocumented
    def _run(self) -> None: ...  # undocumented
    def _stop_platform(self) -> None: ...  # undocumented
    def join(self, *args: Any) -> None: ...

class Events(Generic[_T, _AbstractListener_T]):
    _Listener: type[_AbstractListener_T] | None  # undocumented

    class Event:
        def __eq__(self, other: object) -> bool: ...
    _event_queue: Queue[_T]  # undocumented
    _sentinel: object  # undocumented
    _listener: _AbstractListener_T  # undocumented
    start: Callable[[], None]
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    def __iter__(self: Self) -> Self: ...
    def __next__(self) -> _T: ...
    def get(self, timeout: float | None = ...) -> _T | None: ...
    def _event_mapper(self, event: Callable[_P, object]) -> Callable[_P, None]: ...

class NotifierMixin: ...



from _typeshed import ReadableBuffer, WriteableBuffer
from collections.abc import Iterator
from typing import Any

__all__ = ["calcsize", "pack", "pack_into", "unpack", "unpack_from", "iter_unpack", "Struct", "error"]

class error(Exception): ...

def pack(__fmt: str | bytes, *v: Any) -> bytes: ...
def pack_into(__fmt: str | bytes, __buffer: WriteableBuffer, __offset: int, *v: Any) -> None: ...
def unpack(__format: str | bytes, __buffer: ReadableBuffer) -> tuple[Any, ...]: ...
def unpack_from(__format: str | bytes, buffer: ReadableBuffer, offset: int = ...) -> tuple[Any, ...]: ...
def iter_unpack(__format: str | bytes, __buffer: ReadableBuffer) -> Iterator[tuple[Any, ...]]: ...
def calcsize(__format: str | bytes) -> int: ...

class Struct:
    @property
    def format(self) -> str: ...
    @property
    def size(self) -> int: ...
    def __init__(self, format: str | bytes) -> None: ...
    def pack(self, *v: Any) -> bytes: ...
    def pack_into(self, buffer: WriteableBuffer, offset: int, *v: Any) -> None: ...
    def unpack(self, __buffer: ReadableBuffer) -> tuple[Any, ...]: ...
    def unpack_from(self, buffer: ReadableBuffer, offset: int = ...) -> tuple[Any, ...]: ...
    def iter_unpack(self, __buffer: ReadableBuffer) -> Iterator[tuple[Any, ...]]: ...



from _typeshed import Incomplete
from typing import Any

from pyVmomi.vim import ManagedEntity
from pyVmomi.vim.view import ContainerView
from pyVmomi.vmodl import DynamicProperty

class PropertyCollector:
    class PropertySpec:
        def __init__(self, *, all: bool = ..., type: type[ManagedEntity] = ..., pathSet: list[str] = ...) -> None: ...
        all: bool
        type: type[ManagedEntity]
        pathSet: list[str]

    class TraversalSpec:
        def __init__(
            self, *, path: str = ..., skip: bool = ..., type: type[ContainerView] = ..., **kwargs: Incomplete
        ) -> None: ...
        path: str
        skip: bool
        type: type[ContainerView]
        def __getattr__(self, name: str) -> Incomplete: ...

    class RetrieveOptions:
        def __init__(self, *, maxObjects: int | None = ...) -> None: ...
        maxObjects: int | None

    class ObjectSpec:
        def __init__(
            self, *, skip: bool = ..., selectSet: list[PropertyCollector.TraversalSpec] = ..., obj: Any = ...
        ) -> None: ...
        skip: bool
        selectSet: list[PropertyCollector.TraversalSpec]
        obj: Any

    class FilterSpec:
        def __init__(
            self,
            *,
            propSet: list[PropertyCollector.PropertySpec] = ...,
            objectSet: list[PropertyCollector.ObjectSpec] = ...,
            **kwargs: Incomplete,
        ) -> None: ...
        propSet: list[PropertyCollector.PropertySpec]
        objectSet: list[PropertyCollector.ObjectSpec]
        def __getattr__(self, name: str) -> Incomplete: ...

    class ObjectContent:
        def __init__(self, *, obj: ManagedEntity = ..., propSet: list[DynamicProperty] = ..., **kwargs: Incomplete) -> None: ...
        obj: ManagedEntity
        propSet: list[DynamicProperty]
        def __getattr__(self, name: str) -> Incomplete: ...

    class RetrieveResult:
        def __init__(self, *, objects: list[PropertyCollector.ObjectContent] = ..., token: str | None = ...) -> None: ...
        objects: list[PropertyCollector.ObjectContent]
        token: str | None
    def RetrievePropertiesEx(
        self, specSet: list[PropertyCollector.FilterSpec], options: PropertyCollector.RetrieveOptions
    ) -> PropertyCollector.RetrieveResult: ...
    def ContinueRetrievePropertiesEx(self, token: str) -> PropertyCollector.RetrieveResult: ...
    def __getattr__(self, name: str) -> Incomplete: ...



from typing import Any

from .base import BaseEndpoint as BaseEndpoint

log: Any

class MetadataEndpoint(BaseEndpoint):
    raise_errors: Any
    endpoints: Any
    initial_claims: Any
    claims: Any
    def __init__(self, endpoints, claims=..., raise_errors: bool = ...) -> None: ...
    def create_metadata_response(self, uri, http_method: str = ..., body: Any | None = ..., headers: Any | None = ...): ...
    def validate_metadata(
        self, array, key, is_required: bool = ..., is_list: bool = ..., is_url: bool = ..., is_issuer: bool = ...
    ) -> None: ...
    def validate_metadata_token(self, claims, endpoint) -> None: ...
    def validate_metadata_authorization(self, claims, endpoint): ...
    def validate_metadata_revocation(self, claims, endpoint) -> None: ...
    def validate_metadata_introspection(self, claims, endpoint) -> None: ...
    def validate_metadata_server(self): ...



import distutils.command.install_scripts as orig
from typing import Any

class install_scripts(orig.install_scripts):
    no_ep: bool
    def initialize_options(self) -> None: ...
    outfiles: Any
    def run(self) -> None: ...
    def write_script(self, script_name, contents, mode: str = ..., *ignored) -> None: ...



from __future__ import annotations

import asyncio


class Waiter:
    def __init__(self) -> None:
        self.tasks: list[asyncio.Task[object]] = []

    def add(self, t: asyncio.Task[object]) -> None:
        self.tasks.append(t)

    async def join(self) -> None:
        await asyncio.wait(self.tasks)


async def foo() -> int:
    return 42


async def main() -> None:
    # asyncio.Task is covariant in its type argument, which is unusual since its parent class
    # asyncio.Future is invariant in its type argument. This is only sound because asyncio.Task
    # is not actually Liskov substitutable for asyncio.Future: it does not implement set_result.
    w = Waiter()
    t: asyncio.Task[int] = asyncio.create_task(foo())
    w.add(t)
    await w.join()



from typing import Any

from .base import BaseEndpoint as BaseEndpoint

log: Any

class TokenEndpoint(BaseEndpoint):
    valid_request_methods: Any
    def __init__(self, default_grant_type, default_token_type, grant_types) -> None: ...
    @property
    def grant_types(self): ...
    @property
    def default_grant_type(self): ...
    @property
    def default_grant_type_handler(self): ...
    @property
    def default_token_type(self): ...
    def create_token_response(
        self,
        uri,
        http_method: str = ...,
        body: Any | None = ...,
        headers: Any | None = ...,
        credentials: Any | None = ...,
        grant_type_for_scope: Any | None = ...,
        claims: Any | None = ...,
    ): ...
    def validate_token_request(self, request) -> None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

attribute_mapping: Any

class SeriesLabel(Serialisable):
    tagname: str
    strRef: Any
    v: Any
    value: Any
    __elements__: Any
    def __init__(self, strRef: Any | None = ..., v: Any | None = ...) -> None: ...

class Series(Serialisable):
    tagname: str
    idx: Any
    order: Any
    tx: Any
    title: Any
    spPr: Any
    graphicalProperties: Any
    pictureOptions: Any
    dPt: Any
    data_points: Any
    dLbls: Any
    labels: Any
    trendline: Any
    errBars: Any
    cat: Any
    identifiers: Any
    val: Any
    extLst: Any
    invertIfNegative: Any
    shape: Any
    xVal: Any
    yVal: Any
    bubbleSize: Any
    zVal: Any
    bubble3D: Any
    marker: Any
    smooth: Any
    explosion: Any
    __elements__: Any
    def __init__(
        self,
        idx: int = ...,
        order: int = ...,
        tx: Any | None = ...,
        spPr: Any | None = ...,
        pictureOptions: Any | None = ...,
        dPt=...,
        dLbls: Any | None = ...,
        trendline: Any | None = ...,
        errBars: Any | None = ...,
        cat: Any | None = ...,
        val: Any | None = ...,
        invertIfNegative: Any | None = ...,
        shape: Any | None = ...,
        xVal: Any | None = ...,
        yVal: Any | None = ...,
        bubbleSize: Any | None = ...,
        bubble3D: Any | None = ...,
        marker: Any | None = ...,
        smooth: Any | None = ...,
        explosion: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...
    def to_tree(self, tagname: Any | None = ..., idx: Any | None = ...): ...  # type: ignore[override]

class XYSeries(Series):
    idx: Any
    order: Any
    tx: Any
    spPr: Any
    dPt: Any
    dLbls: Any
    trendline: Any
    errBars: Any
    xVal: Any
    yVal: Any
    invertIfNegative: Any
    bubbleSize: Any
    bubble3D: Any
    marker: Any
    smooth: Any



from typing import Any

from .sync import SyncStrategy

class RestartableStrategy(SyncStrategy):
    sync: bool
    no_real_dsa: bool
    pooled: bool
    can_stream: bool
    restartable_sleep_time: Any
    restartable_tries: Any
    exception_history: Any
    def __init__(self, ldap_connection) -> None: ...
    def open(self, reset_usage: bool = ..., read_server_info: bool = ...) -> None: ...
    def send(self, message_type, request, controls: Any | None = ...): ...
    def post_send_single_response(self, message_id): ...
    def post_send_search(self, message_id): ...
    def get_stream(self) -> None: ...
    def set_stream(self, value) -> None: ...



from typing import Any

class TranslatorError(Exception): ...

class Translator:
    tokenizer: Any
    def __init__(self, formula, origin) -> None: ...
    def get_tokens(self): ...
    ROW_RANGE_RE: Any
    COL_RANGE_RE: Any
    CELL_REF_RE: Any
    @staticmethod
    def translate_row(row_str, rdelta): ...
    @staticmethod
    def translate_col(col_str, cdelta): ...
    @staticmethod
    def strip_ws_name(range_str): ...
    @classmethod
    def translate_range(cls, range_str, rdelta, cdelta): ...
    def translate_formula(self, dest: Any | None = ..., row_delta: int = ..., col_delta: int = ...): ...



from stripe.api_resources.abstract.api_resource import APIResource as APIResource

class SingletonAPIResource(APIResource):
    @classmethod
    def retrieve(cls, **params): ...
    @classmethod
    def class_url(cls): ...
    def instance_url(self): ...



import sys
from _typeshed import structseq
from collections.abc import Callable
from threading import Thread
from types import TracebackType
from typing import Any, NoReturn
from typing_extensions import Final, final

error = RuntimeError

def _count() -> int: ...
@final
class LockType:
    def acquire(self, blocking: bool = ..., timeout: float = ...) -> bool: ...
    def release(self) -> None: ...
    def locked(self) -> bool: ...
    def __enter__(self) -> bool: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...

def start_new_thread(function: Callable[..., object], args: tuple[Any, ...], kwargs: dict[str, Any] = ...) -> int: ...
def interrupt_main() -> None: ...
def exit() -> NoReturn: ...
def allocate_lock() -> LockType: ...
def get_ident() -> int: ...
def stack_size(size: int = ...) -> int: ...

TIMEOUT_MAX: float

if sys.version_info >= (3, 8):
    def get_native_id() -> int: ...  # only available on some platforms
    @final
    class _ExceptHookArgs(structseq[Any], tuple[type[BaseException], BaseException | None, TracebackType | None, Thread | None]):
        if sys.version_info >= (3, 10):
            __match_args__: Final = ("exc_type", "exc_value", "exc_traceback", "thread")
        @property
        def exc_type(self) -> type[BaseException]: ...
        @property
        def exc_value(self) -> BaseException | None: ...
        @property
        def exc_traceback(self) -> TracebackType | None: ...
        @property
        def thread(self) -> Thread | None: ...
    _excepthook: Callable[[_ExceptHookArgs], Any]



from typing import Any
from warnings import warn as warn

class StyleDescriptor:
    collection: Any
    key: Any
    def __init__(self, collection, key) -> None: ...
    def __set__(self, instance, value) -> None: ...
    def __get__(self, instance, cls): ...

class NumberFormatDescriptor:
    key: str
    collection: str
    def __set__(self, instance, value) -> None: ...
    def __get__(self, instance, cls): ...

class NamedStyleDescriptor:
    key: str
    collection: str
    def __set__(self, instance, value) -> None: ...
    def __get__(self, instance, cls): ...

class StyleArrayDescriptor:
    key: Any
    def __init__(self, key) -> None: ...
    def __set__(self, instance, value) -> None: ...
    def __get__(self, instance, cls): ...

class StyleableObject:
    font: Any
    fill: Any
    border: Any
    number_format: Any
    protection: Any
    alignment: Any
    style: Any
    quotePrefix: Any
    pivotButton: Any
    parent: Any
    def __init__(self, sheet, style_array: Any | None = ...) -> None: ...
    @property
    def style_id(self): ...
    @property
    def has_style(self): ...



from typing import Any

class TreeBuilderRegistry:
    builders_for_feature: Any
    builders: Any
    def __init__(self) -> None: ...
    def register(self, treebuilder_class) -> None: ...
    def lookup(self, *features): ...

class TreeBuilder:
    NAME: str
    ALTERNATE_NAMES: Any
    features: Any
    is_xml: bool
    picklable: bool
    empty_element_tags: Any
    DEFAULT_CDATA_LIST_ATTRIBUTES: Any
    DEFAULT_PRESERVE_WHITESPACE_TAGS: Any
    DEFAULT_STRING_CONTAINERS: Any
    USE_DEFAULT: Any
    TRACKS_LINE_NUMBERS: bool
    soup: Any
    cdata_list_attributes: Any
    preserve_whitespace_tags: Any
    store_line_numbers: Any
    string_containers: Any
    def __init__(
        self, multi_valued_attributes=..., preserve_whitespace_tags=..., store_line_numbers=..., string_containers=...
    ) -> None: ...
    def initialize_soup(self, soup) -> None: ...
    def reset(self) -> None: ...
    def can_be_empty_element(self, tag_name): ...
    def feed(self, markup) -> None: ...
    def prepare_markup(
        self,
        markup,
        user_specified_encoding: Any | None = ...,
        document_declared_encoding: Any | None = ...,
        exclude_encodings: Any | None = ...,
    ) -> None: ...
    def test_fragment_to_document(self, fragment): ...
    def set_up_substitutions(self, tag): ...

class SAXTreeBuilder(TreeBuilder):
    def feed(self, markup) -> None: ...
    def close(self) -> None: ...
    def startElement(self, name, attrs) -> None: ...
    def endElement(self, name) -> None: ...
    def startElementNS(self, nsTuple, nodeName, attrs) -> None: ...
    def endElementNS(self, nsTuple, nodeName) -> None: ...
    def startPrefixMapping(self, prefix, nodeValue) -> None: ...
    def endPrefixMapping(self, prefix) -> None: ...
    def characters(self, content) -> None: ...
    def startDocument(self) -> None: ...
    def endDocument(self) -> None: ...

class HTMLTreeBuilder(TreeBuilder):
    empty_element_tags: Any
    block_elements: Any
    DEFAULT_STRING_CONTAINERS: Any
    DEFAULT_CDATA_LIST_ATTRIBUTES: Any
    DEFAULT_PRESERVE_WHITESPACE_TAGS: Any
    def set_up_substitutions(self, tag): ...

class ParserRejectedMarkup(Exception):
    def __init__(self, message_or_exception) -> None: ...

class XMLParsedAsHTMLWarning(UserWarning): ...



from typing import Any, overload

_defaultaction: str
_onceregistry: dict[Any, Any]
filters: list[tuple[str, str | None, type[Warning], str | None, int]]

@overload
def warn(message: str, category: type[Warning] | None = ..., stacklevel: int = ..., source: Any | None = ...) -> None: ...
@overload
def warn(message: Warning, category: Any = ..., stacklevel: int = ..., source: Any | None = ...) -> None: ...
@overload
def warn_explicit(
    message: str,
    category: type[Warning],
    filename: str,
    lineno: int,
    module: str | None = ...,
    registry: dict[str | tuple[str, type[Warning], int], int] | None = ...,
    module_globals: dict[str, Any] | None = ...,
    source: Any | None = ...,
) -> None: ...
@overload
def warn_explicit(
    message: Warning,
    category: Any,
    filename: str,
    lineno: int,
    module: str | None = ...,
    registry: dict[str | tuple[str, type[Warning], int], int] | None = ...,
    module_globals: dict[str, Any] | None = ...,
    source: Any | None = ...,
) -> None: ...



from collections.abc import Callable
from types import coroutine as coroutine
from typing import TypeVar

_T = TypeVar("_T", bound=type)

# All these function actually mutate the given type:
def delegate_to_executor(*attrs: str) -> Callable[[_T], _T]: ...
def proxy_method_directly(*attrs: str) -> Callable[[_T], _T]: ...
def proxy_property_directly(*attrs: str) -> Callable[[_T], _T]: ...
def cond_delegate_to_executor(*attrs: str) -> Callable[[_T], _T]: ...



import http.client
from typing import Any

HAVE_HTTPS_CONNECTION: bool
ON_APP_ENGINE: Any
PORTS_BY_SECURITY: Any
DEFAULT_CA_CERTS_FILE: Any

class HostConnectionPool:
    queue: Any
    def __init__(self) -> None: ...
    def size(self): ...
    def put(self, conn): ...
    def get(self): ...
    def clean(self): ...

class ConnectionPool:
    CLEAN_INTERVAL: float
    STALE_DURATION: float
    host_to_pool: Any
    last_clean_time: float
    mutex: Any
    def __init__(self) -> None: ...
    def size(self): ...
    def get_http_connection(self, host, port, is_secure): ...
    def put_http_connection(self, host, port, is_secure, conn): ...
    def clean(self): ...

class HTTPRequest:
    method: Any
    protocol: Any
    host: Any
    port: Any
    path: Any
    auth_path: Any
    params: Any
    headers: Any
    body: Any
    def __init__(self, method, protocol, host, port, path, auth_path, params, headers, body) -> None: ...
    def authorize(self, connection, **kwargs): ...

class HTTPResponse(http.client.HTTPResponse):
    def __init__(self, *args, **kwargs) -> None: ...
    def read(self, amt: Any | None = ...): ...

class AWSAuthConnection:
    suppress_consec_slashes: Any
    num_retries: int
    is_secure: Any
    https_validate_certificates: Any
    ca_certificates_file: Any
    port: Any
    http_exceptions: Any
    http_unretryable_exceptions: Any
    socket_exception_values: Any
    https_connection_factory: Any
    protocol: str
    host: Any
    path: Any
    debug: Any
    host_header: Any
    http_connection_kwargs: Any
    provider: Any
    auth_service_name: Any
    request_hook: Any
    def __init__(
        self,
        host,
        aws_access_key_id: Any | None = ...,
        aws_secret_access_key: Any | None = ...,
        is_secure: bool = ...,
        port: Any | None = ...,
        proxy: Any | None = ...,
        proxy_port: Any | None = ...,
        proxy_user: Any | None = ...,
        proxy_pass: Any | None = ...,
        debug: int = ...,
        https_connection_factory: Any | None = ...,
        path: str = ...,
        provider: str = ...,
        security_token: Any | None = ...,
        suppress_consec_slashes: bool = ...,
        validate_certs: bool = ...,
        profile_name: Any | None = ...,
    ) -> None: ...
    auth_region_name: Any
    @property
    def connection(self): ...
    @property
    def aws_access_key_id(self): ...
    @property
    def gs_access_key_id(self) -> Any: ...
    @property
    def access_key(self): ...
    @property
    def aws_secret_access_key(self): ...
    @property
    def gs_secret_access_key(self): ...
    @property
    def secret_key(self): ...
    @property
    def profile_name(self): ...
    def get_path(self, path: str = ...): ...
    def server_name(self, port: Any | None = ...): ...
    proxy: Any
    proxy_port: Any
    proxy_user: Any
    proxy_pass: Any
    no_proxy: Any
    use_proxy: Any
    def handle_proxy(self, proxy, proxy_port, proxy_user, proxy_pass): ...
    def get_http_connection(self, host, port, is_secure): ...
    def skip_proxy(self, host): ...
    def new_http_connection(self, host, port, is_secure): ...
    def put_http_connection(self, host, port, is_secure, connection): ...
    def proxy_ssl(self, host: Any | None = ..., port: Any | None = ...): ...
    def prefix_proxy_to_path(self, path, host: Any | None = ...): ...
    def get_proxy_auth_header(self): ...
    def get_proxy_url_with_auth(self): ...
    def set_host_header(self, request): ...
    def set_request_hook(self, hook): ...
    def build_base_http_request(
        self,
        method,
        path,
        auth_path,
        params: Any | None = ...,
        headers: Any | None = ...,
        data: str = ...,
        host: Any | None = ...,
    ): ...
    def make_request(
        self,
        method,
        path,
        headers: Any | None = ...,
        data: str = ...,
        host: Any | None = ...,
        auth_path: Any | None = ...,
        sender: Any | None = ...,
        override_num_retries: Any | None = ...,
        params: Any | None = ...,
        retry_handler: Any | None = ...,
    ): ...
    def close(self): ...

class AWSQueryConnection(AWSAuthConnection):
    APIVersion: str
    ResponseError: Any
    def __init__(
        self,
        aws_access_key_id: Any | None = ...,
        aws_secret_access_key: Any | None = ...,
        is_secure: bool = ...,
        port: Any | None = ...,
        proxy: Any | None = ...,
        proxy_port: Any | None = ...,
        proxy_user: Any | None = ...,
        proxy_pass: Any | None = ...,
        host: Any | None = ...,
        debug: int = ...,
        https_connection_factory: Any | None = ...,
        path: str = ...,
        security_token: Any | None = ...,
        validate_certs: bool = ...,
        profile_name: Any | None = ...,
        provider: str = ...,
    ) -> None: ...
    def get_utf8_value(self, value): ...
    def make_request(  # type: ignore[override]
        self, action, params: Any | None = ..., path: str = ..., verb: str = ..., *args, **kwargs
    ): ...
    def build_list_params(self, params, items, label): ...
    def build_complex_list_params(self, params, items, label, names): ...
    def get_list(self, action, params, markers, path: str = ..., parent: Any | None = ..., verb: str = ...): ...
    def get_object(self, action, params, cls, path: str = ..., parent: Any | None = ..., verb: str = ...): ...
    def get_status(self, action, params, path: str = ..., parent: Any | None = ..., verb: str = ...): ...



import sys
from _typeshed import FileDescriptorLike, ReadOnlyBuffer, WriteableBuffer
from typing import Any, overload
from typing_extensions import Literal

if sys.platform != "win32":
    FASYNC: int
    FD_CLOEXEC: int
    F_DUPFD: int
    F_DUPFD_CLOEXEC: int
    F_GETFD: int
    F_GETFL: int
    F_GETLK: int
    F_GETOWN: int
    F_RDLCK: int
    F_SETFD: int
    F_SETFL: int
    F_SETLK: int
    F_SETLKW: int
    F_SETOWN: int
    F_UNLCK: int
    F_WRLCK: int
    if sys.platform == "darwin":
        F_FULLFSYNC: int
        F_NOCACHE: int
        if sys.version_info >= (3, 9):
            F_GETPATH: int
    if sys.platform == "linux":
        F_SETLKW64: int
        F_SETSIG: int
        F_SHLCK: int
        F_SETLK64: int
        F_SETLEASE: int
        F_GETSIG: int
        F_NOTIFY: int
        F_EXLCK: int
        F_GETLEASE: int
        F_GETLK64: int
        if sys.version_info >= (3, 8):
            F_ADD_SEALS: int
            F_GET_SEALS: int
            F_SEAL_GROW: int
            F_SEAL_SEAL: int
            F_SEAL_SHRINK: int
            F_SEAL_WRITE: int
        if sys.version_info >= (3, 9):
            F_OFD_GETLK: int
            F_OFD_SETLK: int
            F_OFD_SETLKW: int
        if sys.version_info >= (3, 10):
            F_GETPIPE_SZ: int
            F_SETPIPE_SZ: int

        DN_ACCESS: int
        DN_ATTRIB: int
        DN_CREATE: int
        DN_DELETE: int
        DN_MODIFY: int
        DN_MULTISHOT: int
        DN_RENAME: int

    LOCK_EX: int
    LOCK_NB: int
    LOCK_SH: int
    LOCK_UN: int
    if sys.platform == "linux":
        LOCK_MAND: int
        LOCK_READ: int
        LOCK_RW: int
        LOCK_WRITE: int

    # These are highly problematic, they might be present or not, depends on the specific OS.
    if sys.platform == "linux":
        I_ATMARK: int
        I_CANPUT: int
        I_CKBAND: int
        I_FDINSERT: int
        I_FIND: int
        I_FLUSH: int
        I_FLUSHBAND: int
        I_GETBAND: int
        I_GETCLTIME: int
        I_GETSIG: int
        I_GRDOPT: int
        I_GWROPT: int
        I_LINK: int
        I_LIST: int
        I_LOOK: int
        I_NREAD: int
        I_PEEK: int
        I_PLINK: int
        I_POP: int
        I_PUNLINK: int
        I_PUSH: int
        I_RECVFD: int
        I_SENDFD: int
        I_SETCLTIME: int
        I_SETSIG: int
        I_SRDOPT: int
        I_STR: int
        I_SWROPT: int
        I_UNLINK: int
    @overload
    def fcntl(__fd: FileDescriptorLike, __cmd: int, __arg: int = ...) -> int: ...
    @overload
    def fcntl(__fd: FileDescriptorLike, __cmd: int, __arg: str | ReadOnlyBuffer) -> bytes: ...
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: int = ..., __mutate_flag: bool = ...) -> int: ...
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: WriteableBuffer, __mutate_flag: Literal[True] = ...) -> int: ...
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: WriteableBuffer, __mutate_flag: Literal[False]) -> bytes: ...
    @overload
    def ioctl(__fd: FileDescriptorLike, __request: int, __arg: ReadOnlyBuffer, __mutate_flag: bool = ...) -> bytes: ...
    def flock(__fd: FileDescriptorLike, __operation: int) -> None: ...
    def lockf(__fd: FileDescriptorLike, __cmd: int, __len: int = ..., __start: int = ..., __whence: int = ...) -> Any: ...



from ctypes import Structure, _Pointer, c_int32, c_uint, c_void_p
from ctypes.wintypes import FLOAT, UINT

from d3dshot.dll import _HRESULT, _IUnknown
from d3dshot.dll.dxgi import IDXGIAdapter

class DXGI_SAMPLE_DESC(Structure):
    Count: UINT
    Quality: UINT

class D3D11_BOX(Structure):
    left: UINT
    top: UINT
    front: UINT
    right: UINT
    bottom: UINT
    back: UINT

class D3D11_TEXTURE2D_DESC(Structure):
    Width: UINT
    Height: UINT
    MipLevels: UINT
    ArraySize: UINT
    Format: UINT
    SampleDesc: DXGI_SAMPLE_DESC
    Usage: UINT
    BindFlags: UINT
    CPUAccessFlags: UINT
    MiscFlags: UINT

class ID3D11DeviceChild(_IUnknown):
    def GetDevice(self) -> None: ...
    def GetPrivateData(self) -> _HRESULT: ...
    def SetPrivateData(self) -> _HRESULT: ...
    def SetPrivateDataInterface(self) -> _HRESULT: ...

class ID3D11Resource(ID3D11DeviceChild):
    def GetType(self) -> None: ...
    def SetEvictionPriority(self) -> None: ...
    def GetEvictionPriority(self) -> UINT: ...

class ID3D11Texture2D(ID3D11Resource):
    def GetDesc(self, __pDesc: _Pointer[D3D11_TEXTURE2D_DESC]) -> None: ...

class ID3D11DeviceContext(ID3D11DeviceChild):
    def VSSetConstantBuffers(self) -> None: ...
    def PSSetShaderResources(self) -> None: ...
    def PSSetShader(self) -> None: ...
    def PSSetSamplers(self) -> None: ...
    def VSSetShader(self) -> None: ...
    def DrawIndexed(self) -> None: ...
    def Draw(self) -> None: ...
    def Map(self) -> _HRESULT: ...
    def Unmap(self) -> None: ...
    def PSSetConstantBuffers(self) -> None: ...
    def IASetInputLayout(self) -> None: ...
    def IASetVertexBuffers(self) -> None: ...
    def IASetIndexBuffer(self) -> None: ...
    def DrawIndexedInstanced(self) -> None: ...
    def DrawInstanced(self) -> None: ...
    def GSSetConstantBuffers(self) -> None: ...
    def GSSetShader(self) -> None: ...
    def IASetPrimitiveTopology(self) -> None: ...
    def VSSetShaderResources(self) -> None: ...
    def VSSetSamplers(self) -> None: ...
    def Begin(self) -> None: ...
    def End(self) -> None: ...
    def GetData(self) -> _HRESULT: ...
    def SetPredication(self) -> None: ...
    def GSSetShaderResources(self) -> None: ...
    def GSSetSamplers(self) -> None: ...
    def OMSetRenderTargets(self) -> None: ...
    def OMSetRenderTargetsAndUnorderedAccessViews(self) -> None: ...
    def OMSetBlendState(self) -> None: ...
    def OMSetDepthStencilState(self) -> None: ...
    def SOSetTargets(self) -> None: ...
    def DrawAuto(self) -> None: ...
    def DrawIndexedInstancedIndirect(self) -> None: ...
    def DrawInstancedIndirect(self) -> None: ...
    def Dispatch(self) -> None: ...
    def DispatchIndirect(self) -> None: ...
    def RSSetState(self) -> None: ...
    def RSSetViewports(self) -> None: ...
    def RSSetScissorRects(self) -> None: ...
    def CopySubresourceRegion(
        self,
        __pDstResource: _Pointer[ID3D11Resource],
        __DstSubresource: UINT,
        __DstX: UINT,
        __DstY: UINT,
        __DstZ: UINT,
        __pSrcResource: _Pointer[ID3D11Resource],
        __SrcSubresource: UINT,
        __pSrcBox: _Pointer[D3D11_BOX],
    ) -> None: ...
    def CopyResource(self, __pDstResource: _Pointer[ID3D11Resource], __pSrcResource: _Pointer[ID3D11Resource]) -> None: ...
    def UpdateSubresource(self) -> None: ...
    def CopyStructureCount(self) -> None: ...
    def ClearRenderTargetView(self) -> None: ...
    def ClearUnorderedAccessViewUint(self) -> None: ...
    def ClearUnorderedAccessViewFloat(self) -> None: ...
    def ClearDepthStencilView(self) -> None: ...
    def GenerateMips(self) -> None: ...
    def SetResourceMinLOD(self) -> None: ...
    def GetResourceMinLOD(self) -> FLOAT: ...
    def ResolveSubresource(self) -> None: ...
    def ExecuteCommandList(self) -> None: ...
    def HSSetShaderResources(self) -> None: ...
    def HSSetShader(self) -> None: ...
    def HSSetSamplers(self) -> None: ...
    def HSSetConstantBuffers(self) -> None: ...
    def DSSetShaderResources(self) -> None: ...
    def DSSetShader(self) -> None: ...
    def DSSetSamplers(self) -> None: ...
    def DSSetConstantBuffers(self) -> None: ...
    def CSSetShaderResources(self) -> None: ...
    def CSSetUnorderedAccessViews(self) -> None: ...
    def CSSetShader(self) -> None: ...
    def CSSetSamplers(self) -> None: ...
    def CSSetConstantBuffers(self) -> None: ...
    def VSGetConstantBuffers(self) -> None: ...
    def PSGetShaderResources(self) -> None: ...
    def PSGetShader(self) -> None: ...
    def PSGetSamplers(self) -> None: ...
    def VSGetShader(self) -> None: ...
    def PSGetConstantBuffers(self) -> None: ...
    def IAGetInputLayout(self) -> None: ...
    def IAGetVertexBuffers(self) -> None: ...
    def IAGetIndexBuffer(self) -> None: ...
    def GSGetConstantBuffers(self) -> None: ...
    def GSGetShader(self) -> None: ...
    def IAGetPrimitiveTopology(self) -> None: ...
    def VSGetShaderResources(self) -> None: ...
    def VSGetSamplers(self) -> None: ...
    def GetPredication(self) -> None: ...
    def GSGetShaderResources(self) -> None: ...
    def GSGetSamplers(self) -> None: ...
    def OMGetRenderTargets(self) -> None: ...
    def OMGetRenderTargetsAndUnorderedAccessViews(self) -> None: ...
    def OMGetBlendState(self) -> None: ...
    def OMGetDepthStencilState(self) -> None: ...
    def SOGetTargets(self) -> None: ...
    def RSGetState(self) -> None: ...
    def RSGetViewports(self) -> None: ...
    def RSGetScissorRects(self) -> None: ...
    def HSGetShaderResources(self) -> None: ...
    def HSGetShader(self) -> None: ...
    def HSGetSamplers(self) -> None: ...
    def HSGetConstantBuffers(self) -> None: ...
    def DSGetShaderResources(self) -> None: ...
    def DSGetShader(self) -> None: ...
    def DSGetSamplers(self) -> None: ...
    def DSGetConstantBuffers(self) -> None: ...
    def CSGetShaderResources(self) -> None: ...
    def CSGetUnorderedAccessViews(self) -> None: ...
    def CSGetShader(self) -> None: ...
    def CSGetSamplers(self) -> None: ...
    def CSGetConstantBuffers(self) -> None: ...
    def ClearState(self) -> None: ...
    def Flush(self) -> None: ...
    def GetType(self) -> None: ...
    def GetContextFlags(self) -> UINT: ...
    def FinishCommandList(self) -> _HRESULT: ...

class ID3D11Device(_IUnknown):
    def CreateBuffer(self) -> _HRESULT: ...
    def CreateTexture1D(self) -> _HRESULT: ...
    def CreateTexture2D(
        self,
        __pDesc: _Pointer[D3D11_TEXTURE2D_DESC],
        __pInitialData: c_void_p,
        __ppTexture2D: _Pointer[_Pointer[ID3D11Texture2D]],
    ) -> _HRESULT: ...
    def CreateTexture3D(self) -> _HRESULT: ...
    def CreateShaderResourceView(self) -> _HRESULT: ...
    def CreateUnorderedAccessView(self) -> _HRESULT: ...
    def CreateRenderTargetView(self) -> _HRESULT: ...
    def CreateDepthStencilView(self) -> _HRESULT: ...
    def CreateInputLayout(self) -> _HRESULT: ...
    def CreateVertexShader(self) -> _HRESULT: ...
    def CreateGeometryShader(self) -> _HRESULT: ...
    def CreateGeometryShaderWithStreamOutput(self) -> _HRESULT: ...
    def CreatePixelShader(self) -> _HRESULT: ...
    def CreateHullShader(self) -> _HRESULT: ...
    def CreateDomainShader(self) -> _HRESULT: ...
    def CreateComputeShader(self) -> _HRESULT: ...
    def CreateClassLinkage(self) -> _HRESULT: ...
    def CreateBlendState(self) -> _HRESULT: ...
    def CreateDepthStencilState(self) -> _HRESULT: ...
    def CreateRasterizerState(self) -> _HRESULT: ...
    def CreateSamplerState(self) -> _HRESULT: ...
    def CreateQuery(self) -> _HRESULT: ...
    def CreatePredicate(self) -> _HRESULT: ...
    def CreateCounter(self) -> _HRESULT: ...
    def CreateDeferredContext(self) -> _HRESULT: ...
    def OpenSharedResource(self) -> _HRESULT: ...
    def CheckFormatSupport(self) -> _HRESULT: ...
    def CheckMultisampleQualityLevels(self) -> _HRESULT: ...
    def CheckCounterInfo(self) -> _HRESULT: ...
    def CheckCounter(self) -> _HRESULT: ...
    def CheckFeatureSupport(self) -> _HRESULT: ...
    def GetPrivateData(self) -> _HRESULT: ...
    def SetPrivateData(self) -> _HRESULT: ...
    def SetPrivateDataInterface(self) -> _HRESULT: ...
    def GetFeatureLevel(self) -> c_int32: ...
    def GetCreationFlags(self) -> c_uint: ...
    def GetDeviceRemovedReason(self) -> _HRESULT: ...
    def GetImmediateContext(self, __ppImmediateContext: _Pointer[_Pointer[ID3D11DeviceContext]]) -> None: ...
    def SetExceptionMode(self) -> _HRESULT: ...
    def GetExceptionMode(self) -> c_uint: ...

def initialize_d3d_device(dxgi_adapter: _Pointer[IDXGIAdapter]) -> tuple[ID3D11Device, ID3D11DeviceContext]: ...
def describe_d3d11_texture_2d(d3d11_texture_2d: ID3D11Texture2D) -> D3D11_TEXTURE2D_DESC: ...
def prepare_d3d11_texture_2d_for_cpu(d3d11_texture_2d: ID3D11Texture2D, d3d_device: ID3D11Device) -> ID3D11Texture2D: ...



from typing import Any
from typing_extensions import TypeAlias

# Enable when pyasn1 gets stubs:
# from pyasn1.type.univ import OctetString, Sequence
_OctetString: TypeAlias = Any
_Sequence: TypeAlias = Any

class UserIdentity(_OctetString):
    tagSet: Any
    encoding: str

class OldPasswd(_OctetString):
    tagSet: Any
    encoding: str

class NewPasswd(_OctetString):
    tagSet: Any
    encoding: str

class GenPasswd(_OctetString):
    tagSet: Any
    encoding: str

class PasswdModifyRequestValue(_Sequence):
    componentType: Any

class PasswdModifyResponseValue(_Sequence):
    componentType: Any



import datetime
import io
import logging.handlers
import subprocess
import time
from collections.abc import Callable, Iterable, Mapping, Sequence
from contextlib import AbstractContextManager
from email.message import Message
from hashlib import _Hash
from typing import IO, Any, TypeVar
from typing_extensions import TypeAlias

import boto.connection

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")

_Provider: TypeAlias = Any  # TODO replace this with boto.provider.Provider once stubs exist
_LockType: TypeAlias = Any  # TODO replace this with _thread.LockType once stubs exist

JSONDecodeError: type[ValueError]
qsa_of_interest: list[str]

def unquote_v(nv: str) -> str | tuple[str, str]: ...
def canonical_string(
    method: str, path: str, headers: Mapping[str, str | None], expires: int | None = ..., provider: _Provider | None = ...
) -> str: ...
def merge_meta(
    headers: Mapping[str, str], metadata: Mapping[str, str], provider: _Provider | None = ...
) -> Mapping[str, str]: ...
def get_aws_metadata(headers: Mapping[str, str], provider: _Provider | None = ...) -> Mapping[str, str]: ...
def retry_url(url: str, retry_on_404: bool = ..., num_retries: int = ..., timeout: int | None = ...) -> str: ...

class LazyLoadMetadata(dict[_KT, _VT]):
    def __init__(self, url: str, num_retries: int, timeout: int | None = ...) -> None: ...

def get_instance_metadata(
    version: str = ..., url: str = ..., data: str = ..., timeout: int | None = ..., num_retries: int = ...
) -> LazyLoadMetadata[Any, Any] | None: ...
def get_instance_identity(
    version: str = ..., url: str = ..., timeout: int | None = ..., num_retries: int = ...
) -> Mapping[str, Any] | None: ...
def get_instance_userdata(
    version: str = ..., sep: str | None = ..., url: str = ..., timeout: int | None = ..., num_retries: int = ...
) -> Mapping[str, str]: ...

ISO8601: str
ISO8601_MS: str
RFC1123: str
LOCALE_LOCK: _LockType

def setlocale(name: str | tuple[str, str]) -> AbstractContextManager[str]: ...
def get_ts(ts: time.struct_time | None = ...) -> str: ...
def parse_ts(ts: str) -> datetime.datetime: ...
def find_class(module_name: str, class_name: str | None = ...) -> type[Any] | None: ...
def update_dme(username: str, password: str, dme_id: str, ip_address: str) -> str: ...
def fetch_file(
    uri: str, file: IO[str] | None = ..., username: str | None = ..., password: str | None = ...
) -> IO[str] | None: ...

class ShellCommand:
    exit_code: int
    command: subprocess._CMD
    log_fp: io.StringIO
    wait: bool
    fail_fast: bool
    def __init__(
        self, command: subprocess._CMD, wait: bool = ..., fail_fast: bool = ..., cwd: subprocess._TXT | None = ...
    ) -> None: ...
    process: subprocess.Popen[Any]
    def run(self, cwd: subprocess._CMD | None = ...) -> int | None: ...
    def setReadOnly(self, value) -> None: ...
    def getStatus(self) -> int | None: ...
    status: int | None
    def getOutput(self) -> str: ...
    output: str

class AuthSMTPHandler(logging.handlers.SMTPHandler):
    username: str
    password: str
    def __init__(
        self, mailhost: str, username: str, password: str, fromaddr: str, toaddrs: Sequence[str], subject: str
    ) -> None: ...

class LRUCache(dict[_KT, _VT]):
    class _Item:
        previous: LRUCache._Item | None
        next: LRUCache._Item | None
        key = ...
        value = ...
        def __init__(self, key, value) -> None: ...
    _dict: dict[_KT, LRUCache._Item]
    capacity: int
    head: LRUCache._Item | None
    tail: LRUCache._Item | None
    def __init__(self, capacity: int) -> None: ...

# This exists to work around Password.str's name shadowing the str type
_Str: TypeAlias = str

class Password:
    hashfunc: Callable[[bytes], _Hash]
    str: _Str | None
    def __init__(self, str: _Str | None = ..., hashfunc: Callable[[bytes], _Hash] | None = ...) -> None: ...
    def set(self, value: bytes | _Str) -> None: ...
    def __eq__(self, other: _Str | bytes | None) -> bool: ...  # type: ignore[override]
    def __len__(self) -> int: ...

def notify(
    subject: str,
    body: str | None = ...,
    html_body: Sequence[str] | str | None = ...,
    to_string: str | None = ...,
    attachments: Iterable[Message] | None = ...,
    append_instance_id: bool = ...,
) -> None: ...
def get_utf8_value(value: str) -> bytes: ...
def mklist(value: Any) -> list[Any]: ...
def pythonize_name(name: str) -> str: ...
def write_mime_multipart(
    content: list[tuple[str, str]], compress: bool = ..., deftype: str = ..., delimiter: str = ...
) -> str: ...
def guess_mime_type(content: str, deftype: str) -> str: ...
def compute_md5(fp: IO[Any], buf_size: int = ..., size: int | None = ...) -> tuple[str, str, int]: ...
def compute_hash(fp: IO[Any], buf_size: int = ..., size: int | None = ..., hash_algorithm: Any = ...) -> tuple[str, str, int]: ...
def find_matching_headers(name: str, headers: Mapping[str, str | None]) -> list[str]: ...
def merge_headers_by_name(name: str, headers: Mapping[str, str | None]) -> str: ...

class RequestHook:
    def handle_request_data(
        self, request: boto.connection.HTTPRequest, response: boto.connection.HTTPResponse, error: bool = ...
    ) -> Any: ...

def host_is_ipv6(hostname: str) -> bool: ...
def parse_host(hostname: str) -> str: ...



# https://pyinstaller.org/en/stable/hooks.html?highlight=conda_support#module-PyInstaller.utils.hooks.conda

import sys
from _typeshed import StrOrBytesPath
from collections.abc import Iterable
from pathlib import Path
from typing import Any
from typing_extensions import TypeAlias, TypedDict

if sys.version_info >= (3, 8):
    from importlib.metadata import PackagePath as _PackagePath
else:
    _PackagePath: TypeAlias = Any

CONDA_ROOT: Path
CONDA_META_DIR: Path
PYTHONPATH_PREFIXES: list[Path]

class _RawDict(TypedDict):
    name: str
    version: str
    files: list[StrOrBytesPath]
    depends: list[str]

class Distribution:
    raw: _RawDict
    name: str
    version: str
    files: list[PackagePath]
    dependencies: list[str]
    packages: list[str]
    def __init__(self, json_path: str) -> None: ...
    @classmethod
    def from_name(cls, name: str) -> Distribution: ...
    @classmethod
    def from_package_name(cls, name: str) -> Distribution: ...

# distribution and package_distribution are meant to be used and are not internal helpers
distribution = Distribution.from_name
package_distribution = Distribution.from_package_name

class PackagePath(_PackagePath):
    def locate(self) -> Path: ...

def walk_dependency_tree(initial: str, excludes: Iterable[str] | None = ...) -> dict[str, Distribution]: ...
def requires(name: str, strip_versions: bool = ...) -> list[str]: ...
def files(name: str, dependencies: bool = ..., excludes: Iterable[str] | None = ...) -> list[PackagePath]: ...
def collect_dynamic_libs(
    name: str, dest: str = ..., dependencies: bool = ..., excludes: Iterable[str] | None = ...
) -> list[tuple[str, str]]: ...

distributions: dict[str, Distribution]
distributions_by_package: dict[str | None, Distribution]



import datetime
from _typeshed import Incomplete, Self
from abc import ABCMeta, abstractmethod
from collections.abc import Generator, Iterable, Sequence
from enum import Enum
from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
from typing import Any, ClassVar, Generic, TypeVar

from cryptography.hazmat.backends.interfaces import X509Backend
from cryptography.hazmat.primitives.asymmetric.dsa import DSAPrivateKey, DSAPublicKey
from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePrivateKey, EllipticCurvePublicKey
from cryptography.hazmat.primitives.asymmetric.ed448 import Ed448PrivateKey, Ed448PublicKey
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey, Ed25519PublicKey
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
from cryptography.hazmat.primitives.hashes import HashAlgorithm
from cryptography.hazmat.primitives.serialization import Encoding

class ObjectIdentifier:
    @property
    def dotted_string(self) -> str: ...
    def __init__(self, dotted_string: str) -> None: ...

class CRLEntryExtensionOID:
    CERTIFICATE_ISSUER: ClassVar[ObjectIdentifier]
    CRL_REASON: ClassVar[ObjectIdentifier]
    INVALIDITY_DATE: ClassVar[ObjectIdentifier]

class ExtensionOID:
    AUTHORITY_INFORMATION_ACCESS: ClassVar[ObjectIdentifier]
    AUTHORITY_KEY_IDENTIFIER: ClassVar[ObjectIdentifier]
    BASIC_CONSTRAINTS: ClassVar[ObjectIdentifier]
    CERTIFICATE_POLICIES: ClassVar[ObjectIdentifier]
    CRL_DISTRIBUTION_POINTS: ClassVar[ObjectIdentifier]
    CRL_NUMBER: ClassVar[ObjectIdentifier]
    DELTA_CRL_INDICATOR: ClassVar[ObjectIdentifier]
    EXTENDED_KEY_USAGE: ClassVar[ObjectIdentifier]
    FRESHEST_CRL: ClassVar[ObjectIdentifier]
    INHIBIT_ANY_POLICY: ClassVar[ObjectIdentifier]
    ISSUER_ALTERNATIVE_NAME: ClassVar[ObjectIdentifier]
    ISSUING_DISTRIBUTION_POINT: ClassVar[ObjectIdentifier]
    KEY_USAGE: ClassVar[ObjectIdentifier]
    NAME_CONSTRAINTS: ClassVar[ObjectIdentifier]
    OCSP_NO_CHECK: ClassVar[ObjectIdentifier]
    POLICY_CONSTRAINTS: ClassVar[ObjectIdentifier]
    POLICY_MAPPINGS: ClassVar[ObjectIdentifier]
    PRECERT_POISON: ClassVar[ObjectIdentifier]
    PRECERT_SIGNED_CERTIFICATE_TIMESTAMPS: ClassVar[ObjectIdentifier]
    SUBJECT_ALTERNATIVE_NAME: ClassVar[ObjectIdentifier]
    SUBJECT_DIRECTORY_ATTRIBUTES: ClassVar[ObjectIdentifier]
    SUBJECT_INFORMATION_ACCESS: ClassVar[ObjectIdentifier]
    SUBJECT_KEY_IDENTIFIER: ClassVar[ObjectIdentifier]
    TLS_FEATURE: ClassVar[ObjectIdentifier]

class NameOID:
    BUSINESS_CATEGORY: ClassVar[ObjectIdentifier]
    COMMON_NAME: ClassVar[ObjectIdentifier]
    COUNTRY_NAME: ClassVar[ObjectIdentifier]
    DN_QUALIFIER: ClassVar[ObjectIdentifier]
    DOMAIN_COMPONENT: ClassVar[ObjectIdentifier]
    EMAIL_ADDRESS: ClassVar[ObjectIdentifier]
    GENERATION_QUALIFIER: ClassVar[ObjectIdentifier]
    GIVEN_NAME: ClassVar[ObjectIdentifier]
    JURISDICTION_COUNTRY_NAME: ClassVar[ObjectIdentifier]
    JURISDICTION_LOCALITY_NAME: ClassVar[ObjectIdentifier]
    JURISDICTION_STATE_OR_PROVINCE_NAME: ClassVar[ObjectIdentifier]
    LOCALITY_NAME: ClassVar[ObjectIdentifier]
    ORGANIZATIONAL_UNIT_NAME: ClassVar[ObjectIdentifier]
    ORGANIZATION_NAME: ClassVar[ObjectIdentifier]
    POSTAL_ADDRESS: ClassVar[ObjectIdentifier]
    POSTAL_CODE: ClassVar[ObjectIdentifier]
    PSEUDONYM: ClassVar[ObjectIdentifier]
    SERIAL_NUMBER: ClassVar[ObjectIdentifier]
    STATE_OR_PROVINCE_NAME: ClassVar[ObjectIdentifier]
    STREET_ADDRESS: ClassVar[ObjectIdentifier]
    SURNAME: ClassVar[ObjectIdentifier]
    TITLE: ClassVar[ObjectIdentifier]
    USER_ID: ClassVar[ObjectIdentifier]
    X500_UNIQUE_IDENTIFIER: ClassVar[ObjectIdentifier]

class OCSPExtensionOID:
    NONCE: ClassVar[ObjectIdentifier]

class SignatureAlgorithmOID:
    DSA_WITH_SHA1: ClassVar[ObjectIdentifier]
    DSA_WITH_SHA224: ClassVar[ObjectIdentifier]
    DSA_WITH_SHA256: ClassVar[ObjectIdentifier]
    ECDSA_WITH_SHA1: ClassVar[ObjectIdentifier]
    ECDSA_WITH_SHA224: ClassVar[ObjectIdentifier]
    ECDSA_WITH_SHA256: ClassVar[ObjectIdentifier]
    ECDSA_WITH_SHA384: ClassVar[ObjectIdentifier]
    ECDSA_WITH_SHA512: ClassVar[ObjectIdentifier]
    ED25519: ClassVar[ObjectIdentifier]
    ED448: ClassVar[ObjectIdentifier]
    RSASSA_PSS: ClassVar[ObjectIdentifier]
    RSA_WITH_MD5: ClassVar[ObjectIdentifier]
    RSA_WITH_SHA1: ClassVar[ObjectIdentifier]
    RSA_WITH_SHA224: ClassVar[ObjectIdentifier]
    RSA_WITH_SHA256: ClassVar[ObjectIdentifier]
    RSA_WITH_SHA384: ClassVar[ObjectIdentifier]
    RSA_WITH_SHA512: ClassVar[ObjectIdentifier]

class ExtendedKeyUsageOID:
    SERVER_AUTH: ClassVar[ObjectIdentifier]
    CLIENT_AUTH: ClassVar[ObjectIdentifier]
    CODE_SIGNING: ClassVar[ObjectIdentifier]
    EMAIL_PROTECTION: ClassVar[ObjectIdentifier]
    TIME_STAMPING: ClassVar[ObjectIdentifier]
    OCSP_SIGNING: ClassVar[ObjectIdentifier]
    ANY_EXTENDED_KEY_USAGE: ClassVar[ObjectIdentifier]

class NameAttribute:
    @property
    def oid(self) -> ObjectIdentifier: ...
    @property
    def value(self) -> str: ...
    def __init__(self, oid: ObjectIdentifier, value: str) -> None: ...
    def rfc4514_string(self) -> str: ...

class RelativeDistinguishedName:
    def __init__(self, attributes: list[NameAttribute]) -> None: ...
    def __iter__(self) -> Generator[NameAttribute, None, None]: ...
    def get_attributes_for_oid(self, oid: ObjectIdentifier) -> list[NameAttribute]: ...
    def rfc4514_string(self) -> str: ...

class Name:
    @property
    def rdns(self) -> list[RelativeDistinguishedName]: ...
    def __init__(self, attributes: Sequence[NameAttribute | RelativeDistinguishedName]) -> None: ...
    def __iter__(self) -> Generator[NameAttribute, None, None]: ...
    def __len__(self) -> int: ...
    def get_attributes_for_oid(self, oid: ObjectIdentifier) -> list[NameAttribute]: ...
    def public_bytes(self, backend: X509Backend | None = ...) -> bytes: ...
    def rfc4514_string(self) -> str: ...

class Version(Enum):
    v1: int
    v3: int

# These are actually abstractproperties on Certificate,
# but let's not worry too much about that
class Certificate(metaclass=ABCMeta):
    @property
    def extensions(self) -> Extensions: ...
    @property
    def issuer(self) -> Name: ...
    @property
    def not_valid_after(self) -> datetime.datetime: ...
    @property
    def not_valid_before(self) -> datetime.datetime: ...
    @property
    def serial_number(self) -> int: ...
    @property
    def signature(self) -> bytes: ...
    @property
    def signature_algorithm_oid(self) -> ObjectIdentifier: ...
    @property
    def signature_hash_algorithm(self) -> HashAlgorithm: ...
    @property
    def tbs_certificate_bytes(self) -> bytes: ...
    @property
    def subject(self) -> Name: ...
    @property
    def version(self) -> Version: ...
    @abstractmethod
    def fingerprint(self, algorithm: HashAlgorithm) -> bytes: ...
    @abstractmethod
    def public_bytes(self, encoding: Encoding) -> bytes: ...
    @abstractmethod
    def public_key(self) -> DSAPublicKey | Ed25519PublicKey | Ed448PublicKey | EllipticCurvePublicKey | RSAPublicKey: ...
    @abstractmethod
    def __eq__(self, __other: object) -> bool: ...
    @abstractmethod
    def __ne__(self, __other: object) -> bool: ...

class CertificateBuilder:
    def __init__(
        self,
        issuer_name: Name | None = ...,
        subject_name: Name | None = ...,
        public_key: DSAPublicKey | Ed25519PublicKey | Ed448PublicKey | EllipticCurvePublicKey | RSAPublicKey | None = ...,
        serial_number: int | None = ...,
        not_valid_before: datetime.datetime | None = ...,
        not_valid_after: datetime.datetime | None = ...,
        extensions: Iterable[ExtensionType] | None = ...,
    ) -> None: ...
    def add_extension(self, extension: ExtensionType, critical: bool) -> CertificateBuilder: ...
    def issuer_name(self, name: Name) -> CertificateBuilder: ...
    def not_valid_after(self, time: datetime.datetime) -> CertificateBuilder: ...
    def not_valid_before(self, time: datetime.datetime) -> CertificateBuilder: ...
    def public_key(
        self, public_key: DSAPublicKey | Ed25519PublicKey | Ed448PublicKey | EllipticCurvePublicKey | RSAPublicKey
    ) -> CertificateBuilder: ...
    def serial_number(self, serial_number: int) -> CertificateBuilder: ...
    def sign(
        self,
        private_key: DSAPrivateKey | Ed25519PrivateKey | Ed448PrivateKey | EllipticCurvePrivateKey | RSAPrivateKey,
        algorithm: HashAlgorithm | None,
        backend: X509Backend | None = ...,
    ) -> Certificate: ...
    def subject_name(self, name: Name) -> CertificateBuilder: ...

class CertificateRevocationList(metaclass=ABCMeta):
    @property
    def extensions(self) -> Extensions: ...
    @property
    def issuer(self) -> Name: ...
    @property
    def last_update(self) -> datetime.datetime: ...
    @property
    def next_update(self) -> datetime.datetime: ...
    @property
    def signature(self) -> bytes: ...
    @property
    def signature_algorithm_oid(self) -> ObjectIdentifier: ...
    @property
    def signature_hash_algorithm(self) -> HashAlgorithm: ...
    @property
    def tbs_certlist_bytes(self) -> bytes: ...
    @abstractmethod
    def fingerprint(self, algorithm: HashAlgorithm) -> bytes: ...
    @abstractmethod
    def get_revoked_certificate_by_serial_number(self, serial_number: int) -> RevokedCertificate: ...
    @abstractmethod
    def is_signature_valid(
        self, public_key: DSAPublicKey | Ed25519PublicKey | Ed448PublicKey | EllipticCurvePublicKey | RSAPublicKey
    ) -> bool: ...
    @abstractmethod
    def public_bytes(self, encoding: Encoding) -> bytes: ...
    @abstractmethod
    def __eq__(self, __other: object) -> bool: ...
    @abstractmethod
    def __ne__(self, __other: object) -> bool: ...

class CertificateRevocationListBuilder:
    def add_extension(self, extension: ExtensionType, critical: bool) -> CertificateRevocationListBuilder: ...
    def add_revoked_certificate(self, revoked_certificate: RevokedCertificate) -> CertificateRevocationListBuilder: ...
    def issuer_name(self, name: Name) -> CertificateRevocationListBuilder: ...
    def last_update(self, time: datetime.datetime) -> CertificateRevocationListBuilder: ...
    def next_update(self, time: datetime.datetime) -> CertificateRevocationListBuilder: ...
    def sign(
        self,
        private_key: DSAPrivateKey | Ed25519PrivateKey | Ed448PrivateKey | EllipticCurvePrivateKey | RSAPrivateKey,
        algorithm: HashAlgorithm | None,
        backend: X509Backend | None = ...,
    ) -> CertificateRevocationList: ...

class CertificateSigningRequest(metaclass=ABCMeta):
    @property
    def extensions(self) -> Extensions: ...
    @property
    def is_signature_valid(self) -> bool: ...
    @property
    def signature(self) -> bytes: ...
    @property
    def signature_algorithm_oid(self) -> ObjectIdentifier: ...
    @property
    def signature_hash_algorithm(self) -> HashAlgorithm: ...
    @property
    def subject(self) -> Name: ...
    @property
    def tbs_certrequest_bytes(self) -> bytes: ...
    @abstractmethod
    def public_bytes(self, encoding: Encoding) -> bytes: ...
    @abstractmethod
    def public_key(self) -> DSAPublicKey | Ed25519PublicKey | Ed448PublicKey | EllipticCurvePublicKey | RSAPublicKey: ...
    @abstractmethod
    def __eq__(self, __other: object) -> bool: ...
    @abstractmethod
    def __ne__(self, __other: object) -> bool: ...

class CertificateSigningRequestBuilder:
    def add_extension(self, extension: ExtensionType, critical: bool) -> CertificateSigningRequestBuilder: ...
    def subject_name(self, name: Name) -> CertificateSigningRequestBuilder: ...
    def sign(
        self,
        private_key: DSAPrivateKey | Ed25519PrivateKey | Ed448PrivateKey | EllipticCurvePrivateKey | RSAPrivateKey,
        algorithm: HashAlgorithm | None,
        backend: X509Backend | None = ...,
    ) -> CertificateSigningRequest: ...

class RevokedCertificate(metaclass=ABCMeta):
    @property
    def extensions(self) -> Extensions: ...
    @property
    def revocation_date(self) -> datetime.datetime: ...
    @property
    def serial_number(self) -> int: ...

class RevokedCertificateBuilder:
    def add_extension(self, extension: ExtensionType, critical: bool) -> RevokedCertificateBuilder: ...
    def build(self, backend: X509Backend | None = ...) -> RevokedCertificate: ...
    def revocation_date(self, time: datetime.datetime) -> RevokedCertificateBuilder: ...
    def serial_number(self, serial_number: int) -> RevokedCertificateBuilder: ...

# General Name Classes

class GeneralName(metaclass=ABCMeta):
    @property
    def value(self): ...

class DirectoryName(GeneralName):
    @property
    def value(self) -> Name: ...
    def __init__(self, value: Name) -> None: ...

class DNSName(GeneralName):
    @property
    def value(self) -> str: ...
    def __init__(self, value: str) -> None: ...

class IPAddress(GeneralName):
    @property
    def value(self) -> IPv4Address | IPv6Address | IPv4Network | IPv6Network: ...
    def __init__(self, value: IPv4Address | IPv6Address | IPv4Network | IPv6Network) -> None: ...

class OtherName(GeneralName):
    @property
    def type_id(self) -> ObjectIdentifier: ...
    @property
    def value(self) -> bytes: ...
    def __init__(self, type_id: ObjectIdentifier, value: bytes) -> None: ...

class RegisteredID(GeneralName):
    @property
    def value(self) -> ObjectIdentifier: ...
    def __init__(self, value: ObjectIdentifier) -> None: ...

class RFC822Name(GeneralName):
    @property
    def value(self) -> str: ...
    def __init__(self, value: str) -> None: ...

class UniformResourceIdentifier(GeneralName):
    @property
    def value(self) -> str: ...
    def __init__(self, value: str) -> None: ...

# X.509 Extensions

class ExtensionType(metaclass=ABCMeta):
    oid: ObjectIdentifier

_T = TypeVar("_T", bound=ExtensionType)

class Extension(Generic[_T]):
    @property
    def critical(self) -> bool: ...
    @property
    def oid(self) -> ObjectIdentifier: ...
    @property
    def value(self) -> _T: ...

class Extensions:
    def __init__(self, general_names: list[Extension[Any]]) -> None: ...
    def __iter__(self) -> Generator[Extension[Any], None, None]: ...
    def get_extension_for_oid(self, oid: ObjectIdentifier) -> Extension[Any]: ...
    def get_extension_for_class(self, extclass: type[_T]) -> Extension[_T]: ...

class DuplicateExtension(Exception):
    oid: ObjectIdentifier
    def __init__(self, msg: str, oid: ObjectIdentifier) -> None: ...

class ExtensionNotFound(Exception):
    oid: ObjectIdentifier
    def __init__(self, msg: str, oid: ObjectIdentifier) -> None: ...

class IssuerAlternativeName(ExtensionType):
    def __init__(self, general_names: list[GeneralName]) -> None: ...
    def __iter__(self) -> Generator[GeneralName, None, None]: ...
    def get_values_for_type(self, type: type[GeneralName]) -> list[Any]: ...

class SubjectAlternativeName(ExtensionType):
    def __init__(self, general_names: list[GeneralName]) -> None: ...
    def __iter__(self) -> Generator[GeneralName, None, None]: ...
    def get_values_for_type(self, type: type[GeneralName]) -> list[Any]: ...

class AuthorityKeyIdentifier(ExtensionType):
    @property
    def key_identifier(self) -> bytes: ...
    @property
    def authority_cert_issuer(self) -> list[GeneralName] | None: ...
    @property
    def authority_cert_serial_number(self) -> int | None: ...
    def __init__(
        self, key_identifier: bytes, authority_cert_issuer: Iterable[GeneralName] | None, authority_cert_serial_number: int | None
    ) -> None: ...
    @classmethod
    def from_issuer_public_key(
        cls: type[Self], public_key: RSAPublicKey | DSAPublicKey | EllipticCurvePublicKey | Ed25519PublicKey | Ed448PublicKey
    ) -> Self: ...
    @classmethod
    def from_issuer_subject_key_identifier(cls: type[Self], ski: SubjectKeyIdentifier) -> Self: ...

class SubjectKeyIdentifier(ExtensionType):
    @property
    def digest(self) -> bytes: ...
    def __init__(self, digest: bytes) -> None: ...
    @classmethod
    def from_public_key(
        cls: type[Self], public_key: RSAPublicKey | DSAPublicKey | EllipticCurvePublicKey | Ed25519PublicKey | Ed448PublicKey
    ) -> Self: ...

class AccessDescription:
    @property
    def access_method(self) -> ObjectIdentifier: ...
    @property
    def access_location(self) -> GeneralName: ...
    def __init__(self, access_method: ObjectIdentifier, access_location: GeneralName) -> None: ...

class AuthorityInformationAccess(ExtensionType):
    def __init__(self, descriptions: Iterable[AccessDescription]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Generator[AccessDescription, None, None]: ...
    def __getitem__(self, item: int) -> AccessDescription: ...

class SubjectInformationAccess(ExtensionType):
    def __init__(self, descriptions: Iterable[AccessDescription]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Generator[AccessDescription, None, None]: ...
    def __getitem__(self, item: int) -> AccessDescription: ...

class BasicConstraints(ExtensionType):
    @property
    def ca(self) -> bool: ...
    @property
    def path_length(self) -> int | None: ...
    def __init__(self, ca: bool, path_length: int | None) -> None: ...

class KeyUsage(ExtensionType):
    @property
    def digital_signature(self) -> bool: ...
    @property
    def content_commitment(self) -> bool: ...
    @property
    def key_encipherment(self) -> bool: ...
    @property
    def data_encipherment(self) -> bool: ...
    @property
    def key_agreement(self) -> bool: ...
    @property
    def key_cert_sign(self) -> bool: ...
    @property
    def crl_sign(self) -> bool: ...
    @property
    def encipher_only(self) -> bool: ...
    @property
    def decipher_only(self) -> bool: ...
    def __init__(
        self,
        digital_signature: bool,
        content_commitment: bool,
        key_encipherment: bool,
        data_encipherment: bool,
        key_agreement: bool,
        key_cert_sign: bool,
        crl_sign: bool,
        encipher_only: bool,
        decipher_only: bool,
    ) -> None: ...

class ExtendedKeyUsage(ExtensionType):
    def __init__(self, usages: Iterable[ObjectIdentifier]) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Generator[ObjectIdentifier, None, None]: ...
    def __getitem__(self, item: int) -> ObjectIdentifier: ...

class UnrecognizedExtension(ExtensionType):
    @property
    def value(self) -> bytes: ...
    @property
    def oid(self) -> ObjectIdentifier: ...  # type: ignore[override]
    def __init__(self, oid: ObjectIdentifier, value: bytes) -> None: ...

def load_der_x509_certificate(data: bytes, backend: X509Backend | None = ...) -> Certificate: ...
def load_pem_x509_certificate(data: bytes, backend: X509Backend | None = ...) -> Certificate: ...
def load_der_x509_crl(data: bytes, backend: X509Backend | None = ...) -> CertificateRevocationList: ...
def load_pem_x509_crl(data: bytes, backend: X509Backend | None = ...) -> CertificateRevocationList: ...
def load_der_x509_csr(data: bytes, backend: X509Backend | None = ...) -> CertificateSigningRequest: ...
def load_pem_x509_csr(data: bytes, backend: X509Backend | None = ...) -> CertificateSigningRequest: ...
def __getattr__(name: str) -> Incomplete: ...



#!/usr/bin/env python3
import os
import sys

import tomli

distributions = sys.argv[1:]
if not distributions:
    distributions = os.listdir("stubs")

for distribution in distributions:
    with open(f"stubs/{distribution}/METADATA.toml", "rb") as file:
        for apt_package in tomli.load(file).get("tool", {}).get("stubtest", {}).get("apt_dependencies", []):
            print(apt_package)



from asyncio import iscoroutinefunction
from collections.abc import Awaitable, Callable, Coroutine
from typing import Any, Union
from typing_extensions import assert_type


def test_iscoroutinefunction(
    x: Callable[[str, int], Coroutine[str, int, bytes]],
    y: Callable[[str, int], Awaitable[bytes]],
    z: Callable[[str, int], Union[str, Awaitable[bytes]]],
    xx: object,
) -> None:

    if iscoroutinefunction(x):
        assert_type(x, Callable[[str, int], Coroutine[str, int, bytes]])

    if iscoroutinefunction(y):
        assert_type(y, Callable[[str, int], Coroutine[Any, Any, bytes]])

    if iscoroutinefunction(z):
        assert_type(z, Callable[[str, int], Coroutine[Any, Any, Any]])

    if iscoroutinefunction(xx):
        assert_type(xx, Callable[..., Coroutine[Any, Any, Any]])



from collections.abc import Generator
from typing import Any

class MQTTMatcher:
    class Node: ...

    def __init__(self) -> None: ...
    def __setitem__(self, key: str, value: Any) -> None: ...
    def __getitem__(self, key: str) -> Any: ...
    def __delitem__(self, key: str) -> None: ...
    def iter_match(self, topic: str) -> Generator[Any, None, None]: ...



# Can't generate with stubgen because:
# "KeyError: 'pywintypes'"
from _typeshed import Incomplete
from typing_extensions import Literal

import _win32typing

class error(Exception): ...
class com_error(Exception): ...

def DosDateTimeToTime() -> _win32typing.PyTime: ...
def Unicode() -> str: ...
def UnicodeFromRaw(_str: str) -> str: ...
def IsTextUnicode(_str: str, flags) -> tuple[Incomplete, Incomplete]: ...
def OVERLAPPED() -> _win32typing.PyOVERLAPPED: ...
def IID(iidString: str, is_bytes: bool = ...) -> _win32typing.PyIID: ...
def Time(timeRepr) -> _win32typing.PyTime: ...
def CreateGuid() -> _win32typing.PyIID: ...
def ACL(bufSize: int = ...) -> _win32typing.PyACL: ...
def SID(buffer, idAuthority, subAuthorities, bufSize=...) -> _win32typing.PySID: ...
def SECURITY_ATTRIBUTES() -> _win32typing.PySECURITY_ATTRIBUTES: ...
def SECURITY_DESCRIPTOR() -> _win32typing.PySECURITY_DESCRIPTOR: ...
def HANDLE() -> int: ...
def HKEY() -> _win32typing.PyHKEY: ...
def WAVEFORMATEX() -> _win32typing.PyWAVEFORMATEX: ...

FALSE: Literal[False]
TRUE: Literal[True]

def TimeStamp(*args, **kwargs): ...  # incomplete



import enum
from collections.abc import Iterable
from typing import Any, NamedTuple

from ._common import (
    ENCODING as ENCODING,
    ENCODING_ERRS as ENCODING_ERRS,
    AccessDenied as AccessDenied,
    NoSuchProcess as NoSuchProcess,
    TimeoutExpired as TimeoutExpired,
    conn_tmap as conn_tmap,
    conn_to_ntuple as conn_to_ntuple,
    debug as debug,
    isfile_strict as isfile_strict,
    memoize as memoize,
    parse_environ_block as parse_environ_block,
    usage_percent as usage_percent,
)
from ._psutil_windows import (
    ABOVE_NORMAL_PRIORITY_CLASS as ABOVE_NORMAL_PRIORITY_CLASS,
    BELOW_NORMAL_PRIORITY_CLASS as BELOW_NORMAL_PRIORITY_CLASS,
    HIGH_PRIORITY_CLASS as HIGH_PRIORITY_CLASS,
    IDLE_PRIORITY_CLASS as IDLE_PRIORITY_CLASS,
    NORMAL_PRIORITY_CLASS as NORMAL_PRIORITY_CLASS,
    REALTIME_PRIORITY_CLASS as REALTIME_PRIORITY_CLASS,
)

msg: str
__extra__all__: Any
CONN_DELETE_TCB: str
ERROR_PARTIAL_COPY: int
PYPY: Any
AF_LINK: int
AddressFamily: Any
TCP_STATUSES: Any

class Priority(enum.IntEnum):
    ABOVE_NORMAL_PRIORITY_CLASS: Any
    BELOW_NORMAL_PRIORITY_CLASS: Any
    HIGH_PRIORITY_CLASS: Any
    IDLE_PRIORITY_CLASS: Any
    NORMAL_PRIORITY_CLASS: Any
    REALTIME_PRIORITY_CLASS: Any

IOPRIO_VERYLOW: int
IOPRIO_LOW: int
IOPRIO_NORMAL: int
IOPRIO_HIGH: int

class IOPriority(enum.IntEnum):
    IOPRIO_VERYLOW: int
    IOPRIO_LOW: int
    IOPRIO_NORMAL: int
    IOPRIO_HIGH: int

pinfo_map: Any

class scputimes(NamedTuple):
    user: Any
    system: Any
    idle: Any
    interrupt: Any
    dpc: Any

class svmem(NamedTuple):
    total: int
    available: int
    percent: float
    used: int
    free: int

class pmem(NamedTuple):
    rss: Any
    vms: Any
    num_page_faults: Any
    peak_wset: Any
    wset: Any
    peak_paged_pool: Any
    paged_pool: Any
    peak_nonpaged_pool: Any
    nonpaged_pool: Any
    pagefile: Any
    peak_pagefile: Any
    private: Any

pfullmem: Any

class pmmap_grouped(NamedTuple):
    path: Any
    rss: Any

pmmap_ext: Any

class pio(NamedTuple):
    read_count: Any
    write_count: Any
    read_bytes: Any
    write_bytes: Any
    other_count: Any
    other_bytes: Any

def convert_dos_path(s): ...
def py2_strencode(s): ...
def getpagesize(): ...
def virtual_memory() -> svmem: ...
def swap_memory(): ...

disk_io_counters: Any

def disk_usage(path): ...
def disk_partitions(all): ...
def cpu_times(): ...
def per_cpu_times(): ...
def cpu_count_logical(): ...
def cpu_count_cores() -> int | None: ...
def cpu_stats(): ...
def cpu_freq(): ...
def getloadavg(): ...
def net_connections(kind, _pid: int = ...): ...
def net_if_stats(): ...
def net_io_counters(): ...
def net_if_addrs(): ...
def sensors_battery(): ...
def boot_time(): ...
def users(): ...
def win_service_iter() -> Iterable[WindowsService]: ...
def win_service_get(name): ...

class WindowsService:
    def __init__(self, name, display_name) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def name(self): ...
    def display_name(self): ...
    def binpath(self): ...
    def username(self): ...
    def start_type(self): ...
    def pid(self): ...
    def status(self): ...
    def description(self): ...
    def as_dict(self): ...

pids: Any
pid_exists: Any
ppid_map: Any

def is_permission_err(exc): ...
def convert_oserror(exc, pid: Any | None = ..., name: Any | None = ...): ...
def wrap_exceptions(fun): ...
def retry_error_partial_copy(fun): ...

class Process:
    pid: Any
    def __init__(self, pid) -> None: ...
    def oneshot_enter(self) -> None: ...
    def oneshot_exit(self) -> None: ...
    def name(self): ...
    def exe(self): ...
    def cmdline(self): ...
    def environ(self): ...
    def ppid(self): ...
    def memory_info(self): ...
    def memory_full_info(self): ...
    def memory_maps(self) -> None: ...
    def kill(self): ...
    def send_signal(self, sig) -> None: ...
    def wait(self, timeout: Any | None = ...): ...
    def username(self): ...
    def create_time(self): ...
    def num_threads(self): ...
    def threads(self): ...
    def cpu_times(self): ...
    def suspend(self) -> None: ...
    def resume(self) -> None: ...
    def cwd(self): ...
    def open_files(self): ...
    def connections(self, kind: str = ...): ...
    def nice_get(self): ...
    def nice_set(self, value): ...
    def ionice_get(self): ...
    def ionice_set(self, ioclass, value) -> None: ...
    def io_counters(self): ...
    def status(self): ...
    def cpu_affinity_get(self): ...
    def cpu_affinity_set(self, value): ...
    def num_handles(self): ...
    def num_ctx_switches(self): ...



def sasl_prep(data): ...
def validate_simple_password(password, accept_empty: bool = ...): ...
def abort_sasl_negotiation(connection, controls): ...
def send_sasl_negotiation(connection, controls, payload): ...
def random_hex_string(size): ...



from datetime import date, datetime, time, timedelta, tzinfo
from typing import Any, overload
from typing_extensions import Literal, TypeAlias

from babel.core import Locale
from babel.util import LOCALTZ as LOCALTZ, UTC as UTC
from pytz import BaseTzInfo

# The module contents here are organized the same way they are in the API documentation at
# http://babel.pocoo.org/en/latest/api/dates.html

# Date and Time Formatting
_Instant: TypeAlias = date | time | datetime | float | None
_PredefinedTimeFormat: TypeAlias = Literal["full", "long", "medium", "short"]

def format_datetime(
    datetime: _Instant = ..., format: _PredefinedTimeFormat | str = ..., tzinfo: tzinfo | None = ..., locale: str | Locale = ...
) -> str: ...
def format_date(
    date: date | datetime | None = ..., format: _PredefinedTimeFormat | str = ..., locale: str | Locale = ...
) -> str: ...
def format_time(
    time: time | datetime | float | None = ...,
    format: _PredefinedTimeFormat | str = ...,
    tzinfo: tzinfo | None = ...,
    locale: str | Locale = ...,
) -> str: ...
def format_timedelta(
    delta: timedelta | int,
    granularity: Literal["year", "month", "week", "day", "hour", "minute", "second"] = ...,
    threshold: float = ...,
    add_direction: bool = ...,
    format: Literal["narrow", "short", "medium", "long"] = ...,
    locale: str | Locale = ...,
) -> str: ...
def format_skeleton(
    skeleton: str, datetime: _Instant = ..., tzinfo: tzinfo | None = ..., fuzzy: bool = ..., locale: str | Locale = ...
) -> str: ...
def format_interval(
    start: _Instant,
    end: _Instant,
    skeleton: str | None = ...,
    tzinfo: tzinfo | None = ...,
    fuzzy: bool = ...,
    locale: str | Locale = ...,
) -> str: ...

# Timezone Functionality
@overload
def get_timezone(zone: str | BaseTzInfo | None = ...) -> BaseTzInfo: ...
@overload
def get_timezone(zone: tzinfo) -> tzinfo: ...
def get_timezone_gmt(
    datetime: _Instant = ...,
    width: Literal["long", "short", "iso8601", "iso8601_short"] = ...,
    locale: str | Locale = ...,
    return_z: bool = ...,
) -> str: ...

_DtOrTzinfo: TypeAlias = datetime | tzinfo | str | int | time | None

def get_timezone_location(dt_or_tzinfo: _DtOrTzinfo = ..., locale: str | Locale = ..., return_city: bool = ...) -> str: ...
def get_timezone_name(
    dt_or_tzinfo: _DtOrTzinfo = ...,
    width: Literal["long", "short"] = ...,
    uncommon: bool = ...,
    locale: str | Locale = ...,
    zone_variant: Literal["generic", "daylight", "standard"] | None = ...,
    return_zone: bool = ...,
) -> str: ...

# Note: While Babel accepts any tzinfo for the most part, the get_next_timeout_transition()
# function requires a tzinfo that is produced by get_timezone()/pytz AND has DST info.
# The typing here will help you with the first requirement, but will not protect against
# pytz tzinfo's without DST info, like what you get from get_timezone("UTC") for instance.
def get_next_timezone_transition(zone: BaseTzInfo | None = ..., dt: _Instant = ...) -> TimezoneTransition: ...

class TimezoneTransition:
    # This class itself is not included in the documentation, yet it is mentioned by name.
    # See https://github.com/python-babel/babel/issues/823
    activates: datetime
    from_tzinfo: tzinfo
    to_tzinfo: tzinfo
    reference_date: datetime | None
    def __init__(
        self, activates: datetime, from_tzinfo: tzinfo, to_tzinfo: tzinfo, reference_date: datetime | None = ...
    ) -> None: ...
    @property
    def from_tz(self) -> str: ...
    @property
    def to_tz(self) -> str: ...
    @property
    def from_offset(self) -> int: ...
    @property
    def to_offset(self) -> int: ...

# Data Access
def get_period_names(width: str = ..., context: str = ..., locale=...): ...
def get_day_names(width: str = ..., context: str = ..., locale=...): ...
def get_month_names(width: str = ..., context: str = ..., locale=...): ...
def get_quarter_names(width: str = ..., context: str = ..., locale=...): ...
def get_era_names(width: str = ..., locale=...): ...
def get_date_format(format: str = ..., locale=...): ...
def get_datetime_format(format: str = ..., locale=...): ...
def get_time_format(format: str = ..., locale=...): ...

# Basic Parsing
def parse_date(string, locale=..., format: str = ...): ...
def parse_time(string, locale=..., format: str = ...): ...
def parse_pattern(pattern): ...

# Undocumented
NO_INHERITANCE_MARKER: str
LC_TIME: Any
date_ = date
datetime_ = datetime
time_ = time

TIMEDELTA_UNITS: Any

def get_period_id(time, tzinfo: Any | None = ..., type: Any | None = ..., locale=...): ...

class DateTimePattern:
    pattern: Any
    format: Any
    def __init__(self, pattern, format) -> None: ...
    def __mod__(self, other): ...
    def apply(self, datetime, locale): ...

class DateTimeFormat:
    value: Any
    locale: Any
    def __init__(self, value, locale) -> None: ...
    def __getitem__(self, name): ...
    def extract(self, char): ...
    def format_era(self, char, num): ...
    def format_year(self, char, num): ...
    def format_quarter(self, char, num): ...
    def format_month(self, char, num): ...
    def format_week(self, char, num): ...
    def format_weekday(self, char: str = ..., num: int = ...): ...
    def format_day_of_year(self, num): ...
    def format_day_of_week_in_month(self): ...
    def format_period(self, char, num): ...
    def format_frac_seconds(self, num): ...
    def format_milliseconds_in_day(self, num): ...
    def format_timezone(self, char, num): ...
    def format(self, value, length): ...
    def get_day_of_year(self, date: Any | None = ...): ...
    def get_week_number(self, day_of_period, day_of_week: Any | None = ...): ...

PATTERN_CHARS: Any
PATTERN_CHAR_ORDER: str

def tokenize_pattern(pattern): ...
def untokenize_pattern(tokens): ...
def split_interval_pattern(pattern): ...
def match_skeleton(skeleton, options, allow_different_fields: bool = ...): ...



from typing import Any

def track_cascade_events(descriptor, prop): ...

class UOWTransaction:
    session: Any
    attributes: Any
    deps: Any
    mappers: Any
    presort_actions: Any
    postsort_actions: Any
    dependencies: Any
    states: Any
    post_update_states: Any
    def __init__(self, session): ...
    @property
    def has_work(self): ...
    def was_already_deleted(self, state): ...
    def is_deleted(self, state): ...
    def memo(self, key, callable_): ...
    def remove_state_actions(self, state) -> None: ...
    def get_attribute_history(self, state, key, passive=...): ...
    def has_dep(self, processor): ...
    def register_preprocessor(self, processor, fromparent) -> None: ...
    def register_object(
        self,
        state,
        isdelete: bool = ...,
        listonly: bool = ...,
        cancel_delete: bool = ...,
        operation: Any | None = ...,
        prop: Any | None = ...,
    ): ...
    def register_post_update(self, state, post_update_cols) -> None: ...
    def filter_states_for_dep(self, dep, states): ...
    def states_for_mapper_hierarchy(self, mapper, isdelete, listonly) -> None: ...
    def execute(self): ...
    def finalize_flush_changes(self) -> None: ...

class IterateMappersMixin: ...

class Preprocess(IterateMappersMixin):
    dependency_processor: Any
    fromparent: Any
    processed: Any
    setup_flush_actions: bool
    def __init__(self, dependency_processor, fromparent) -> None: ...
    def execute(self, uow): ...

class PostSortRec:
    disabled: Any
    # At runtime __new__ is defined, not __init__,
    # But if we define __new__ here then all subclasses are identified by pyright
    # as having __init__ methods that are inconsistent with their __new__ methods
    def __init__(self, uow, *args) -> None: ...
    def execute_aggregate(self, uow, recs) -> None: ...

class ProcessAll(IterateMappersMixin, PostSortRec):
    dependency_processor: Any
    sort_key: Any
    isdelete: Any
    fromparent: Any
    def __init__(self, uow, dependency_processor, isdelete, fromparent) -> None: ...
    def execute(self, uow) -> None: ...
    def per_state_flush_actions(self, uow): ...

class PostUpdateAll(PostSortRec):
    mapper: Any
    isdelete: Any
    sort_key: Any
    def __init__(self, uow, mapper, isdelete) -> None: ...
    def execute(self, uow) -> None: ...

class SaveUpdateAll(PostSortRec):
    mapper: Any
    sort_key: Any
    def __init__(self, uow, mapper) -> None: ...
    def execute(self, uow) -> None: ...
    def per_state_flush_actions(self, uow) -> None: ...

class DeleteAll(PostSortRec):
    mapper: Any
    sort_key: Any
    def __init__(self, uow, mapper) -> None: ...
    def execute(self, uow) -> None: ...
    def per_state_flush_actions(self, uow) -> None: ...

class ProcessState(PostSortRec):
    dependency_processor: Any
    sort_key: Any
    isdelete: Any
    state: Any
    def __init__(self, uow, dependency_processor, isdelete, state) -> None: ...
    def execute_aggregate(self, uow, recs) -> None: ...

class SaveUpdateState(PostSortRec):
    state: Any
    mapper: Any
    sort_key: Any
    def __init__(self, uow, state) -> None: ...
    def execute_aggregate(self, uow, recs) -> None: ...

class DeleteState(PostSortRec):
    state: Any
    mapper: Any
    sort_key: Any
    def __init__(self, uow, state) -> None: ...
    def execute_aggregate(self, uow, recs) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator

from .error import CDefError as CDefError, VerificationError as VerificationError, VerificationMissing as VerificationMissing
from .lock import allocate_lock as allocate_lock

Q_CONST: int
Q_RESTRICT: int
Q_VOLATILE: int

def qualify(quals, replace_with): ...

class BaseTypeByIdentity:
    is_array_type: bool
    is_raw_function: bool
    def get_c_name(self, replace_with: str = ..., context: str = ..., quals: int = ...): ...
    def has_c_name(self): ...
    def is_integer_type(self): ...
    def get_cached_btype(self, ffi, finishlist, can_delay: bool = ...): ...

class BaseType(BaseTypeByIdentity):
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __hash__(self): ...

class VoidType(BaseType):
    c_name_with_marker: str
    def __init__(self) -> None: ...
    def build_backend_type(self, ffi, finishlist): ...

void_type: Incomplete

class BasePrimitiveType(BaseType):
    def is_complex_type(self): ...

class PrimitiveType(BasePrimitiveType):
    ALL_PRIMITIVE_TYPES: Incomplete
    name: Incomplete
    c_name_with_marker: Incomplete
    def __init__(self, name) -> None: ...
    def is_char_type(self): ...
    def is_integer_type(self): ...
    def is_float_type(self): ...
    def is_complex_type(self): ...
    def build_backend_type(self, ffi, finishlist): ...

class UnknownIntegerType(BasePrimitiveType):
    name: Incomplete
    c_name_with_marker: Incomplete
    def __init__(self, name) -> None: ...
    def is_integer_type(self): ...
    def build_backend_type(self, ffi, finishlist) -> None: ...

class UnknownFloatType(BasePrimitiveType):
    name: Incomplete
    c_name_with_marker: Incomplete
    def __init__(self, name) -> None: ...
    def build_backend_type(self, ffi, finishlist) -> None: ...

class BaseFunctionType(BaseType):
    args: Incomplete
    result: Incomplete
    ellipsis: Incomplete
    abi: Incomplete
    c_name_with_marker: Incomplete
    def __init__(self, args, result, ellipsis, abi: Incomplete | None = ...) -> None: ...

class RawFunctionType(BaseFunctionType):
    is_raw_function: bool
    def build_backend_type(self, ffi, finishlist) -> None: ...
    def as_function_pointer(self): ...

class FunctionPtrType(BaseFunctionType):
    def build_backend_type(self, ffi, finishlist): ...
    def as_raw_function(self): ...

class PointerType(BaseType):
    totype: Incomplete
    quals: Incomplete
    c_name_with_marker: Incomplete
    def __init__(self, totype, quals: int = ...) -> None: ...
    def build_backend_type(self, ffi, finishlist): ...

voidp_type: Incomplete

def ConstPointerType(totype): ...

const_voidp_type: Incomplete

class NamedPointerType(PointerType):
    name: Incomplete
    c_name_with_marker: Incomplete
    def __init__(self, totype, name, quals: int = ...) -> None: ...

class ArrayType(BaseType):
    is_array_type: bool
    item: Incomplete
    length: Incomplete
    c_name_with_marker: Incomplete
    def __init__(self, item, length) -> None: ...
    def length_is_unknown(self): ...
    def resolve_length(self, newlength): ...
    def build_backend_type(self, ffi, finishlist): ...

char_array_type: Incomplete

class StructOrUnionOrEnum(BaseTypeByIdentity):
    forcename: Incomplete
    c_name_with_marker: Incomplete
    def build_c_name_with_marker(self) -> None: ...
    def force_the_name(self, forcename) -> None: ...
    def get_official_name(self): ...

class StructOrUnion(StructOrUnionOrEnum):
    fixedlayout: Incomplete
    completed: int
    partial: bool
    packed: int
    name: Incomplete
    fldnames: Incomplete
    fldtypes: Incomplete
    fldbitsize: Incomplete
    fldquals: Incomplete
    def __init__(self, name, fldnames, fldtypes, fldbitsize, fldquals: Incomplete | None = ...) -> None: ...
    def anonymous_struct_fields(self) -> Generator[Incomplete, None, None]: ...
    def enumfields(self, expand_anonymous_struct_union: bool = ...) -> Generator[Incomplete, None, None]: ...
    def force_flatten(self) -> None: ...
    def get_cached_btype(self, ffi, finishlist, can_delay: bool = ...): ...
    def finish_backend_type(self, ffi, finishlist) -> None: ...
    def check_not_partial(self) -> None: ...
    def build_backend_type(self, ffi, finishlist): ...

class StructType(StructOrUnion):
    kind: str

class UnionType(StructOrUnion):
    kind: str

class EnumType(StructOrUnionOrEnum):
    kind: str
    partial: bool
    partial_resolved: bool
    name: Incomplete
    enumerators: Incomplete
    enumvalues: Incomplete
    baseinttype: Incomplete
    def __init__(self, name, enumerators, enumvalues, baseinttype: Incomplete | None = ...) -> None: ...
    forcename: Incomplete
    def force_the_name(self, forcename) -> None: ...
    def check_not_partial(self) -> None: ...
    def build_backend_type(self, ffi, finishlist): ...
    def build_baseinttype(self, ffi, finishlist): ...

def unknown_type(name, structname: Incomplete | None = ...): ...
def unknown_ptr_type(name, structname: Incomplete | None = ...): ...

global_lock: Incomplete

def get_typecache(backend): ...
def global_cache(srctype, ffi, funcname, *args, **kwds): ...
def pointer_cache(ffi, BType): ...
def attach_exception_info(e, name) -> None: ...



import abc
from _typeshed import Self
from typing import Any, ClassVar

from passlib.ifc import PasswordHash
from passlib.utils.binary import BASE64_CHARS, HASH64_CHARS, LOWER_HEX_CHARS, PADDED_BASE64_CHARS, UPPER_HEX_CHARS

H64_CHARS = HASH64_CHARS
B64_CHARS = BASE64_CHARS
PADDED_B64_CHARS = PADDED_BASE64_CHARS
UC_HEX_CHARS = UPPER_HEX_CHARS
LC_HEX_CHARS = LOWER_HEX_CHARS

def parse_mc2(hash, prefix, sep=..., handler: Any | None = ...): ...
def parse_mc3(hash, prefix, sep=..., rounds_base: int = ..., default_rounds: Any | None = ..., handler: Any | None = ...): ...
def render_mc2(ident, salt, checksum, sep=...): ...
def render_mc3(ident, rounds, salt, checksum, sep=..., rounds_base: int = ...): ...

class MinimalHandler(PasswordHash, metaclass=abc.ABCMeta):
    @classmethod
    def using(cls: Self, relaxed: bool = ...) -> type[Self]: ...  # type: ignore[override]

class TruncateMixin(MinimalHandler, metaclass=abc.ABCMeta):
    truncate_error: ClassVar[bool]
    truncate_verify_reject: ClassVar[bool]
    @classmethod
    def using(cls: type[Self], truncate_error: object = ..., *, relaxed: bool = ...) -> type[Self]: ...  # type: ignore[override]

class GenericHandler(MinimalHandler):
    setting_kwds: ClassVar[tuple[str, ...]]
    context_kwds: ClassVar[tuple[str, ...]]
    ident: ClassVar[str | None]
    checksum_size: ClassVar[int | None]
    checksum_chars: ClassVar[str | None]
    checksum: str | None
    use_defaults: bool
    def __init__(self, checksum: str | None = ..., use_defaults: bool = ...) -> None: ...
    @classmethod
    def identify(cls, hash: str | bytes) -> bool: ...
    @classmethod
    def from_string(cls: type[Self], hash: str | bytes, **context: Any) -> Self: ...
    def to_string(self) -> str: ...
    @classmethod
    def hash(cls, secret: str | bytes, **kwds: Any) -> str: ...
    @classmethod
    def verify(cls, secret: str | bytes, hash: str | bytes, **context: Any) -> bool: ...
    @classmethod
    def genconfig(cls, **kwds: Any) -> str: ...
    @classmethod
    def genhash(cls, secret: str | bytes, config: str, **context: Any) -> str: ...
    @classmethod
    def needs_update(cls, hash: str | bytes, secret: str | bytes | None = ..., **kwds: Any) -> bool: ...
    @classmethod
    def parsehash(cls, hash: str | bytes, checksum: bool = ..., sanitize: bool = ...) -> dict[str, Any]: ...
    @classmethod
    def bitsize(cls, **kwds: Any) -> dict[str, Any]: ...

class StaticHandler(GenericHandler):
    setting_kwds: ClassVar[tuple[str, ...]]

class HasEncodingContext(GenericHandler):
    default_encoding: ClassVar[str]
    encoding: str
    def __init__(self, encoding: str | None = ..., **kwds) -> None: ...

class HasUserContext(GenericHandler):
    user: Any | None
    def __init__(self, user: Any | None = ..., **kwds) -> None: ...
    @classmethod
    def hash(cls, secret, user: Any | None = ..., **context): ...
    @classmethod
    def verify(cls, secret, hash, user: Any | None = ..., **context): ...
    @classmethod
    def genhash(cls, secret, config, user: Any | None = ..., **context): ...

class HasRawChecksum(GenericHandler): ...

class HasManyIdents(GenericHandler):
    default_ident: ClassVar[str | None]
    ident_values: ClassVar[tuple[str, ...] | None]
    ident_aliases: ClassVar[dict[str, str] | None]
    ident: str  # type: ignore[misc]
    @classmethod
    def using(cls, default_ident: Any | None = ..., ident: Any | None = ..., **kwds): ...  # type: ignore[override]
    def __init__(self, ident: Any | None = ..., **kwds) -> None: ...

class HasSalt(GenericHandler):
    min_salt_size: ClassVar[int]
    max_salt_size: ClassVar[int | None]
    salt_chars: ClassVar[str | None]
    default_salt_size: ClassVar[int | None]
    default_salt_chars: ClassVar[str | None]
    salt: str | bytes | None
    @classmethod
    def using(cls, default_salt_size: int | None = ..., salt_size: int | None = ..., salt: str | bytes | None = ..., **kwds): ...  # type: ignore[override]
    def __init__(self, salt: str | bytes | None = ..., **kwds) -> None: ...
    @classmethod
    def bitsize(cls, salt_size: int | None = ..., **kwds): ...

class HasRawSalt(HasSalt):
    salt_chars: ClassVar[bytes]  # type: ignore[assignment]

class HasRounds(GenericHandler):
    min_rounds: ClassVar[int]
    max_rounds: ClassVar[int | None]
    rounds_cost: ClassVar[str]
    using_rounds_kwds: ClassVar[tuple[str, ...]]
    min_desired_rounds: ClassVar[int | None]
    max_desired_rounds: ClassVar[int | None]
    default_rounds: ClassVar[int | None]
    vary_rounds: ClassVar[Any | None]
    rounds: int
    @classmethod
    def using(  # type: ignore[override]
        cls,
        min_desired_rounds: Any | None = ...,
        max_desired_rounds: Any | None = ...,
        default_rounds: Any | None = ...,
        vary_rounds: Any | None = ...,
        min_rounds: Any | None = ...,
        max_rounds: Any | None = ...,
        rounds: Any | None = ...,
        **kwds,
    ): ...
    def __init__(self, rounds: Any | None = ..., **kwds) -> None: ...
    @classmethod
    def bitsize(cls, rounds: Any | None = ..., vary_rounds: float = ..., **kwds): ...

class ParallelismMixin(GenericHandler):
    parallelism: int
    @classmethod
    def using(cls, parallelism: Any | None = ..., **kwds): ...  # type: ignore[override]
    def __init__(self, parallelism: Any | None = ..., **kwds) -> None: ...

class BackendMixin(PasswordHash, metaclass=abc.ABCMeta):
    backends: ClassVar[tuple[str, ...] | None]
    @classmethod
    def get_backend(cls): ...
    @classmethod
    def has_backend(cls, name: str = ...) -> bool: ...
    @classmethod
    def set_backend(cls, name: str = ..., dryrun: bool = ...): ...

class SubclassBackendMixin(BackendMixin, metaclass=abc.ABCMeta): ...
class HasManyBackends(BackendMixin, GenericHandler): ...

class PrefixWrapper:
    name: Any
    prefix: Any
    orig_prefix: Any
    __doc__: Any
    def __init__(
        self, name, wrapped, prefix=..., orig_prefix=..., lazy: bool = ..., doc: Any | None = ..., ident: Any | None = ...
    ) -> None: ...
    @property
    def wrapped(self): ...
    @property
    def ident(self): ...
    @property
    def ident_values(self): ...
    def __dir__(self): ...
    def __getattr__(self, attr): ...
    def __setattr__(self, attr, value): ...
    def using(self, **kwds): ...
    def needs_update(self, hash, **kwds): ...
    def identify(self, hash): ...
    def genconfig(self, **kwds): ...
    def genhash(self, secret, config, **kwds): ...
    def encrypt(self, secret, **kwds): ...
    def hash(self, secret, **kwds): ...
    def verify(self, secret, hash, **kwds): ...
